@@ -20,7 +20,12 @@ import logging
 
 from swh.core.db import BaseDb
 from swh.journal.writer.kafka import KafkaJournalWriter
-from swh.storage.converters import db_to_release, db_to_revision
+from swh.storage.converters import (
+    db_to_raw_extrinsic_metadata,
+    db_to_release,
+    db_to_revision,
+)
+from swh.storage.replay import object_converter_fn
 
 
 logger = logging.getLogger(__name__)
@@ -29,11 +34,15 @@ PARTITION_KEY = {
     "content": "sha1",
     "skipped_content": "sha1",
     "directory": "id",
+    "metadata_authority": "type, url",
+    "metadata_fetcher": "name, version",
+    "raw_extrinsic_metadata": "id",
     "revision": "revision.id",
     "release": "release.id",
     "snapshot": "id",
     "origin": "id",
     "origin_visit": "origin_visit.origin",
+    "origin_visit_status": "origin_visit_status.origin",
 }
 
 COLUMNS = {
@@ -57,19 +66,39 @@ COLUMNS = {
         "reason",
     ],
     "directory": ["id", "dir_entries", "file_entries", "rev_entries"],
+    "metadata_authority": ["type", "url", "metadata",],
+    "metadata_fetcher": ["name", "version", "metadata",],
+    "raw_extrinsic_metadata": [
+        "raw_extrinsic_metadata.type",
+        "raw_extrinsic_metadata.id",
+        "metadata_authority.type",
+        "metadata_authority.url",
+        "metadata_fetcher.name",
+        "metadata_fetcher.version",
+        "discovery_date",
+        "format",
+        "raw_extrinsic_metadata.metadata",
+        "origin",
+        "visit",
+        "snapshot",
+        "release",
+        "revision",
+        "path",
+        "directory",
+    ],
     "revision": [
         ("revision.id", "id"),
         "date",
         "date_offset",
+        "date_neg_utc_offset",
         "committer_date",
         "committer_date_offset",
+        "committer_date_neg_utc_offset",
         "type",
         "directory",
         "message",
         "synthetic",
         "metadata",
-        "date_neg_utc_offset",
-        "committer_date_neg_utc_offset",
         (
             "array(select parent_id::bytea from revision_history rh "
             "where rh.id = revision.id order by rh.parent_rank asc)",
@@ -88,10 +117,10 @@ COLUMNS = {
         ("release.id", "id"),
         "date",
         "date_offset",
+        "date_neg_utc_offset",
         "comment",
         ("release.name", "name"),
         "synthetic",
-        "date_neg_utc_offset",
         "target",
         "target_type",
         ("a.id", "author_id"),
@@ -100,12 +129,11 @@ COLUMNS = {
         ("a.fullname", "author_fullname"),
     ],
     "snapshot": ["id", "object_id"],
-    "origin": ["type", "url"],
-    "origin_visit": [
+    "origin": ["url"],
+    "origin_visit": ["visit", "type", ("origin.url", "origin"), "date",],
+    "origin_visit_status": [
         "visit",
-        "origin.type",
-        "origin_visit.type",
-        "url",
+        ("origin.url", "origin"),
         "date",
         "snapshot",
         "status",
@@ -121,6 +149,12 @@ JOINS = {
         "person c on revision.committer=c.id",
     ],
     "origin_visit": ["origin on origin_visit.origin=origin.id"],
+    "origin_visit_status": ["origin on origin_visit_status.origin=origin.id"],
+    "raw_extrinsic_metadata": [
+        "metadata_authority on "
+        "raw_extrinsic_metadata.authority_id=metadata_authority.id",
+        "metadata_fetcher on raw_extrinsic_metadata.fetcher_id=metadata_fetcher.id",
+    ],
 }
 
 
@@ -158,6 +192,14 @@ def directory_converter(db, directory):
     return directory
 
 
+def raw_extrinsic_metadata_converter(db, metadata):
+    """Convert revision from the flat representation to swh model
+       compatible objects.
+
+    """
+    return db_to_raw_extrinsic_metadata(metadata).to_dict()
+
+
 def revision_converter(db, revision):
     """Convert revision from the flat representation to swh model
        compatible objects.
@@ -171,10 +213,7 @@ def release_converter(db, release):
        compatible objects.
 
     """
-    release = db_to_release(release)
-    if "author" in release and release["author"]:
-        del release["author"]["id"]
-    return release
+    return db_to_release(release)
 
 
 def snapshot_converter(db, snapshot):
@@ -204,22 +243,12 @@ def snapshot_converter(db, snapshot):
     return snapshot
 
 
-def origin_visit_converter(db, origin_visit):
-    origin = {
-        "type": origin_visit.pop("origin.type"),
-        "url": origin_visit.pop("url"),
-    }
-    origin_visit["origin"] = origin
-    origin_visit["type"] = origin_visit.pop("origin_visit.type")
-    return origin_visit
-
-
 CONVERTERS = {
     "directory": directory_converter,
+    "raw_extrinsic_metadata": raw_extrinsic_metadata_converter,
     "revision": revision_converter,
     "release": release_converter,
     "snapshot": snapshot_converter,
-    "origin_visit": origin_visit_converter,
 }
 
 
@@ -307,6 +336,7 @@ RANGE_GENERATORS = {
     "snapshot": lambda start, end: byte_ranges(16, start, end),
     "origin": integer_ranges,
     "origin_visit": integer_ranges,
+    "origin_visit_status": integer_ranges,
 }
 
 
@@ -478,7 +508,8 @@ class JournalBackfiller:
                 _format_range_bound(range_end),
             )
 
-            for obj in fetch(db, object_type, start=range_start, end=range_end,):
+            for obj_d in fetch(db, object_type, start=range_start, end=range_end,):
+                obj = object_converter_fn[object_type](obj_d)
                 if dry_run:
                     continue
                 writer.write_addition(object_type=object_type, object_=obj)
