@@ -3,12 +3,13 @@
 # License: GNU General Public License version 3, or any later version
 # See top-level LICENSE file for more information
 
+import base64
 import datetime
 import itertools
 import json
 import random
 import re
-from typing import Any, Dict, List, Iterable, Optional, Tuple, Union
+from typing import Any, Dict, List, Iterable, Optional, Set, Tuple, Union
 
 import attr
 
@@ -31,8 +32,9 @@ from swh.model.model import (
     MetadataFetcher,
     MetadataTargetType,
     RawExtrinsicMetadata,
+    Sha1Git,
 )
-from swh.storage.interface import ListOrder, PagedResult
+from swh.storage.interface import ListOrder, PagedResult, Sha1, VISIT_STATUSES
 from swh.storage.objstorage import ObjStorage
 from swh.storage.writer import JournalWriter
 from swh.storage.utils import map_optional, now
@@ -54,7 +56,7 @@ class CassandraStorage:
         self.journal_writer = JournalWriter(journal_writer)
         self.objstorage = ObjStorage(objstorage)
 
-    def check_config(self, *, check_write):
+    def check_config(self, *, check_write: bool) -> bool:
         self._cql_runner.check_read()
 
         return True
@@ -151,7 +153,9 @@ class CassandraStorage:
         contents = [attr.evolve(c, ctime=now()) for c in content]
         return self._content_add(list(contents), with_data=True)
 
-    def content_update(self, content, keys=[]):
+    def content_update(
+        self, contents: List[Dict[str, Any]], keys: List[str] = []
+    ) -> None:
         raise NotImplementedError(
             "content_update is not supported by the Cassandra backend"
         )
@@ -159,20 +163,17 @@ class CassandraStorage:
     def content_add_metadata(self, content: List[Content]) -> Dict:
         return self._content_add(content, with_data=False)
 
-    def content_get(self, content):
-        if len(content) > BULK_BLOCK_CONTENT_LEN_MAX:
-            raise StorageArgumentException(
-                "Sending at most %s contents." % BULK_BLOCK_CONTENT_LEN_MAX
-            )
-        yield from self.objstorage.content_get(content)
+    def content_get_data(self, content: Sha1) -> Optional[bytes]:
+        # FIXME: Make this method support slicing the `data`
+        return self.objstorage.content_get(content)
 
     def content_get_partition(
         self,
         partition_id: int,
         nb_partitions: int,
+        page_token: Optional[str] = None,
         limit: int = 1000,
-        page_token: str = None,
-    ):
+    ) -> PagedResult[Content]:
         if limit is None:
             raise StorageArgumentException("limit should not be None")
 
@@ -188,39 +189,46 @@ class CassandraStorage:
                 raise StorageArgumentException("Invalid page_token.")
             range_start = int(page_token)
 
-        # Get the first rows of the range
-        rows = self._cql_runner.content_get_token_range(range_start, range_end, limit)
-        rows = list(rows)
+        next_page_token: Optional[str] = None
 
-        if len(rows) == limit:
-            next_page_token: Optional[str] = str(rows[-1].tok + 1)
-        else:
-            next_page_token = None
+        rows = self._cql_runner.content_get_token_range(
+            range_start, range_end, limit + 1
+        )
+        contents = []
+        last_id: Optional[int] = None
+        for counter, row in enumerate(rows):
+            if row.status == "absent":
+                continue
+            row_d = row._asdict()
+            last_id = row_d.pop("tok")
+            if counter >= limit:
+                next_page_token = str(last_id)
+                break
+            contents.append(Content(**row_d))
 
-        return {
-            "contents": [row._asdict() for row in rows if row.status != "absent"],
-            "next_page_token": next_page_token,
-        }
+        assert len(contents) <= limit
+        return PagedResult(results=contents, next_page_token=next_page_token)
 
-    def content_get_metadata(self, contents: List[bytes]) -> Dict[bytes, List[Dict]]:
-        result: Dict[bytes, List[Dict]] = {sha1: [] for sha1 in contents}
+    def content_get(self, contents: List[Sha1]) -> List[Optional[Content]]:
+        contents_by_sha1: Dict[Sha1, Optional[Content]] = {}
         for sha1 in contents:
             # Get all (sha1, sha1_git, sha256, blake2s256) whose sha1
             # matches the argument, from the index table ('content_by_sha1')
             for row in self._content_get_from_hash("sha1", sha1):
-                content_metadata = row._asdict()
-                content_metadata.pop("ctime")
-                result[content_metadata["sha1"]].append(content_metadata)
-        return result
+                row_d = row._asdict()
+                row_d.pop("ctime")
+                content = Content(**row_d)
+                contents_by_sha1[content.sha1] = content
+        return [contents_by_sha1.get(sha1) for sha1 in contents]
 
-    def content_find(self, content):
+    def content_find(self, content: Dict[str, Any]) -> List[Content]:
         # Find an algorithm that is common to all the requested contents.
         # It will be used to do an initial filtering efficiently.
         filter_algos = list(set(content).intersection(HASH_ALGORITHMS))
         if not filter_algos:
             raise StorageArgumentException(
-                "content keys must contain at least one of: "
-                "%s" % ", ".join(sorted(HASH_ALGORITHMS))
+                "content keys must contain at least one "
+                f"of: {', '.join(sorted(HASH_ALGORITHMS))}"
             )
         common_algo = filter_algos[0]
 
@@ -235,31 +243,35 @@ class CassandraStorage:
                     break
             else:
                 # All hashes match, keep this row.
-                results.append(
-                    {
-                        **row._asdict(),
-                        "ctime": row.ctime.replace(tzinfo=datetime.timezone.utc),
-                    }
-                )
+                row_d = row._asdict()
+                row_d["ctime"] = row.ctime.replace(tzinfo=datetime.timezone.utc)
+                results.append(Content(**row_d))
         return results
 
-    def content_missing(self, content, key_hash="sha1"):
-        for cont in content:
-            res = self.content_find(cont)
-            if not res:
-                yield cont[key_hash]
-            if any(c["status"] == "missing" for c in res):
-                yield cont[key_hash]
+    def content_missing(
+        self, contents: List[Dict[str, Any]], key_hash: str = "sha1"
+    ) -> Iterable[bytes]:
+        if key_hash not in DEFAULT_ALGORITHMS:
+            raise StorageArgumentException(
+                "key_hash should be one of {','.join(DEFAULT_ALGORITHMS)}"
+            )
 
-    def content_missing_per_sha1(self, contents):
+        for content in contents:
+            res = self.content_find(content)
+            if not res:
+                yield content[key_hash]
+
+    def content_missing_per_sha1(self, contents: List[bytes]) -> Iterable[bytes]:
         return self.content_missing([{"sha1": c for c in contents}])
 
-    def content_missing_per_sha1_git(self, contents):
+    def content_missing_per_sha1_git(
+        self, contents: List[Sha1Git]
+    ) -> Iterable[Sha1Git]:
         return self.content_missing(
             [{"sha1_git": c for c in contents}], key_hash="sha1_git"
         )
 
-    def content_get_random(self):
+    def content_get_random(self) -> Sha1Git:
         return self._cql_runner.content_get_random().sha1_git
 
     def _skipped_content_get_from_hash(self, algo, hash_) -> Iterable:
@@ -310,7 +322,9 @@ class CassandraStorage:
         contents = [attr.evolve(c, ctime=now()) for c in content]
         return self._skipped_content_add(contents)
 
-    def skipped_content_missing(self, contents):
+    def skipped_content_missing(
+        self, contents: List[Dict[str, Any]]
+    ) -> Iterable[Dict[str, Any]]:
         for content in contents:
             if not self._cql_runner.skipped_content_get_from_pk(content):
                 yield {algo: content[algo] for algo in DEFAULT_ALGORITHMS}
@@ -334,12 +348,12 @@ class CassandraStorage:
             # with half the entries.
             self._cql_runner.directory_add_one(directory.id)
 
-        return {"directory:add": len(missing)}
+        return {"directory:add": len(directories)}
 
-    def directory_missing(self, directories):
+    def directory_missing(self, directories: List[Sha1Git]) -> Iterable[Sha1Git]:
         return self._cql_runner.directory_missing(directories)
 
-    def _join_dentry_to_content(self, dentry):
+    def _join_dentry_to_content(self, dentry: DirectoryEntry) -> Dict[str, Any]:
         keys = (
             "status",
             "sha1",
@@ -350,14 +364,16 @@ class CassandraStorage:
         ret = dict.fromkeys(keys)
         ret.update(dentry.to_dict())
         if ret["type"] == "file":
-            content = self.content_find({"sha1_git": ret["target"]})
-            if content:
-                content = content[0]
+            contents = self.content_find({"sha1_git": ret["target"]})
+            if contents:
+                content = contents[0]
                 for key in keys:
-                    ret[key] = content[key]
+                    ret[key] = getattr(content, key)
         return ret
 
-    def _directory_ls(self, directory_id, recursive, prefix=b""):
+    def _directory_ls(
+        self, directory_id: Sha1Git, recursive: bool, prefix: bytes = b""
+    ) -> Iterable[Dict[str, Any]]:
         if self.directory_missing([directory_id]):
             return
         rows = list(self._cql_runner.directory_entry_get([directory_id]))
@@ -377,17 +393,21 @@ class CassandraStorage:
                     ret["target"], True, prefix + ret["name"] + b"/"
                 )
 
-    def directory_entry_get_by_path(self, directory, paths):
+    def directory_entry_get_by_path(
+        self, directory: Sha1Git, paths: List[bytes]
+    ) -> Optional[Dict[str, Any]]:
         return self._directory_entry_get_by_path(directory, paths, b"")
 
-    def _directory_entry_get_by_path(self, directory, paths, prefix):
+    def _directory_entry_get_by_path(
+        self, directory: Sha1Git, paths: List[bytes], prefix: bytes
+    ) -> Optional[Dict[str, Any]]:
         if not paths:
-            return
+            return None
 
         contents = list(self.directory_ls(directory))
 
         if not contents:
-            return
+            return None
 
         def _get_entry(entries, name):
             """Finds the entry with the requested name, prepends the
@@ -406,16 +426,18 @@ class CassandraStorage:
             return first_item
 
         if not first_item or first_item["type"] != "dir":
-            return
+            return None
 
         return self._directory_entry_get_by_path(
             first_item["target"], paths[1:], prefix + paths[0] + b"/"
         )
 
-    def directory_ls(self, directory, recursive=False):
+    def directory_ls(
+        self, directory: Sha1Git, recursive: bool = False
+    ) -> Iterable[Dict[str, Any]]:
         yield from self._directory_ls(directory, recursive)
 
-    def directory_get_random(self):
+    def directory_get_random(self) -> Sha1Git:
         return self._cql_runner.directory_get_random().id
 
     def revision_add(self, revisions: List[Revision]) -> Dict:
@@ -441,10 +463,12 @@ class CassandraStorage:
 
         return {"revision:add": len(revisions)}
 
-    def revision_missing(self, revisions):
+    def revision_missing(self, revisions: List[Sha1Git]) -> Iterable[Sha1Git]:
         return self._cql_runner.revision_missing(revisions)
 
-    def revision_get(self, revisions):
+    def revision_get(
+        self, revisions: List[Sha1Git]
+    ) -> Iterable[Optional[Dict[str, Any]]]:
         rows = self._cql_runner.revision_get(revisions)
         revs = {}
         for row in rows:
@@ -462,7 +486,15 @@ class CassandraStorage:
         for rev_id in revisions:
             yield revs.get(rev_id)
 
-    def _get_parent_revs(self, rev_ids, seen, limit, short):
+    def _get_parent_revs(
+        self,
+        rev_ids: Iterable[Sha1Git],
+        seen: Set[Sha1Git],
+        limit: Optional[int],
+        short: bool,
+    ) -> Union[
+        Iterable[Dict[str, Any]], Iterable[Tuple[Sha1Git, Tuple[Sha1Git, ...]]],
+    ]:
         if limit and len(seen) >= limit:
             return
         rev_ids = [id_ for id_ in rev_ids if id_ not in seen]
@@ -496,15 +528,19 @@ class CassandraStorage:
                 yield rev.to_dict()
             yield from self._get_parent_revs(parents, seen, limit, short)
 
-    def revision_log(self, revisions, limit=None):
-        seen = set()
+    def revision_log(
+        self, revisions: List[Sha1Git], limit: Optional[int] = None
+    ) -> Iterable[Optional[Dict[str, Any]]]:
+        seen: Set[Sha1Git] = set()
         yield from self._get_parent_revs(revisions, seen, limit, False)
 
-    def revision_shortlog(self, revisions, limit=None):
-        seen = set()
+    def revision_shortlog(
+        self, revisions: List[Sha1Git], limit: Optional[int] = None
+    ) -> Iterable[Optional[Tuple[Sha1Git, Tuple[Sha1Git, ...]]]]:
+        seen: Set[Sha1Git] = set()
         yield from self._get_parent_revs(revisions, seen, limit, True)
 
-    def revision_get_random(self):
+    def revision_get_random(self) -> Sha1Git:
         return self._cql_runner.revision_get_random().id
 
     def release_add(self, releases: List[Release]) -> Dict:
@@ -523,10 +559,12 @@ class CassandraStorage:
 
         return {"release:add": len(to_add)}
 
-    def release_missing(self, releases):
+    def release_missing(self, releases: List[Sha1Git]) -> Iterable[Sha1Git]:
         return self._cql_runner.release_missing(releases)
 
-    def release_get(self, releases):
+    def release_get(
+        self, releases: List[Sha1Git]
+    ) -> Iterable[Optional[Dict[str, Any]]]:
         rows = self._cql_runner.release_get(releases)
         rels = {}
         for row in rows:
@@ -536,7 +574,7 @@ class CassandraStorage:
         for rel_id in releases:
             yield rels.get(rel_id)
 
-    def release_get_random(self):
+    def release_get_random(self) -> Sha1Git:
         return self._cql_runner.release_get_random().id
 
     def snapshot_add(self, snapshots: List[Snapshot]) -> Dict:
@@ -570,21 +608,23 @@ class CassandraStorage:
 
         return {"snapshot:add": len(snapshots)}
 
-    def snapshot_missing(self, snapshots):
+    def snapshot_missing(self, snapshots: List[Sha1Git]) -> Iterable[Sha1Git]:
         return self._cql_runner.snapshot_missing(snapshots)
 
-    def snapshot_get(self, snapshot_id):
+    def snapshot_get(self, snapshot_id: Sha1Git) -> Optional[Dict[str, Any]]:
         return self.snapshot_get_branches(snapshot_id)
 
-    def snapshot_get_by_origin_visit(self, origin, visit):
+    def snapshot_get_by_origin_visit(
+        self, origin: str, visit: int
+    ) -> Optional[Dict[str, Any]]:
         visit_status = self.origin_visit_status_get_latest(
             origin, visit, require_snapshot=True
         )
-        if not visit_status:
-            return None
-        return self.snapshot_get(visit_status.snapshot)
+        if visit_status and visit_status.snapshot:
+            return self.snapshot_get(visit_status.snapshot)
+        return None
 
-    def snapshot_count_branches(self, snapshot_id):
+    def snapshot_count_branches(self, snapshot_id: Sha1Git) -> Optional[Dict[str, int]]:
         if self._cql_runner.snapshot_missing([snapshot_id]):
             # Makes sure we don't fetch branches for a snapshot that is
             # being added.
@@ -598,14 +638,18 @@ class CassandraStorage:
         return counts
 
     def snapshot_get_branches(
-        self, snapshot_id, branches_from=b"", branches_count=1000, target_types=None
-    ):
+        self,
+        snapshot_id: Sha1Git,
+        branches_from: bytes = b"",
+        branches_count: int = 1000,
+        target_types: Optional[List[str]] = None,
+    ) -> Optional[Dict[str, Any]]:
         if self._cql_runner.snapshot_missing([snapshot_id]):
             # Makes sure we don't fetch branches for a snapshot that is
             # being added.
             return None
 
-        branches = []
+        branches: List = []
         while len(branches) < branches_count + 1:
             new_branches = list(
                 self._cql_runner.snapshot_branch_get(
@@ -638,7 +682,7 @@ class CassandraStorage:
         else:
             last_branch = None
 
-        branches = {
+        branches_d = {
             branch.name: {"target": branch.target, "target_type": branch.target_type,}
             if branch.target
             else None
@@ -647,15 +691,15 @@ class CassandraStorage:
 
         return {
             "id": snapshot_id,
-            "branches": branches,
+            "branches": branches_d,
             "next_branch": last_branch,
         }
 
-    def snapshot_get_random(self):
+    def snapshot_get_random(self) -> Sha1Git:
         return self._cql_runner.snapshot_get_random().id
 
-    def object_find_by_sha1_git(self, ids):
-        results = {id_: [] for id_ in ids}
+    def object_find_by_sha1_git(self, ids: List[Sha1Git]) -> Dict[Sha1Git, List[Dict]]:
+        results: Dict[Sha1Git, List[Dict]] = {id_: [] for id_ in ids}
         missing_ids = set(ids)
 
         # Mind the order, revision is the most likely one for a given ID,
@@ -695,14 +739,12 @@ class CassandraStorage:
         else:
             return None
 
-    def origin_get_by_sha1(self, sha1s):
+    def origin_get_by_sha1(self, sha1s: List[bytes]) -> List[Optional[Dict[str, Any]]]:
         results = []
         for sha1 in sha1s:
             rows = self._cql_runner.origin_get_by_sha1(sha1)
-            if rows:
-                results.append({"url": rows.one().url})
-            else:
-                results.append(None)
+            origin = {"url": rows.one().url} if rows else None
+            results.append(origin)
         return results
 
     def origin_list(
@@ -729,25 +771,41 @@ class CassandraStorage:
             # excluding that origin from the result to respect the limit size
             origins = origins[:limit]
 
-        assert (len(origins)) <= limit
+        assert len(origins) <= limit
 
         return PagedResult(results=origins, next_page_token=next_page_token)
 
     def origin_search(
-        self, url_pattern, offset=0, limit=50, regexp=False, with_visit=False
-    ):
+        self,
+        url_pattern: str,
+        page_token: Optional[str] = None,
+        limit: int = 50,
+        regexp: bool = False,
+        with_visit: bool = False,
+    ) -> PagedResult[Origin]:
         # TODO: remove this endpoint, swh-search should be used instead.
+        next_page_token = None
+        offset = int(page_token) if page_token else 0
+
         origins = self._cql_runner.origin_iter_all()
         if regexp:
             pat = re.compile(url_pattern)
-            origins = [orig for orig in origins if pat.search(orig.url)]
+            origins = [Origin(orig.url) for orig in origins if pat.search(orig.url)]
         else:
-            origins = [orig for orig in origins if url_pattern in orig.url]
+            origins = [Origin(orig.url) for orig in origins if url_pattern in orig.url]
 
         if with_visit:
-            origins = [orig for orig in origins if orig.next_visit_id > 1]
+            origins = [Origin(orig.url) for orig in origins if orig.next_visit_id > 1]
 
-        return [{"url": orig.url,} for orig in origins[offset : offset + limit]]
+        origins = origins[offset : offset + limit + 1]
+        if len(origins) > limit:
+            # next offset
+            next_page_token = str(offset + limit)
+            # excluding that origin from the result to respect the limit size
+            origins = origins[:limit]
+
+        assert len(origins) <= limit
+        return PagedResult(results=origins, next_page_token=next_page_token)
 
     def origin_add(self, origins: List[Origin]) -> Dict[str, int]:
         to_add = [ori for ori in origins if self.origin_get_one(ori.url) is None]
@@ -924,6 +982,11 @@ class CassandraStorage:
         allowed_statuses: Optional[List[str]] = None,
         require_snapshot: bool = False,
     ) -> Optional[OriginVisit]:
+        if allowed_statuses and not set(allowed_statuses).intersection(VISIT_STATUSES):
+            raise StorageArgumentException(
+                f"Unknown allowed statuses {','.join(allowed_statuses)}, only "
+                f"{','.join(VISIT_STATUSES)} authorized"
+            )
         # TODO: Do not fetch all visits
         rows = self._cql_runner.origin_visit_get_all(origin)
         latest_visit = None
@@ -962,6 +1025,11 @@ class CassandraStorage:
         allowed_statuses: Optional[List[str]] = None,
         require_snapshot: bool = False,
     ) -> Optional[OriginVisitStatus]:
+        if allowed_statuses and not set(allowed_statuses).intersection(VISIT_STATUSES):
+            raise StorageArgumentException(
+                f"Unknown allowed statuses {','.join(allowed_statuses)}, only "
+                f"{','.join(VISIT_STATUSES)} authorized"
+            )
         rows = self._cql_runner.origin_visit_status_get(
             origin_url, visit, allowed_statuses, require_snapshot
         )
@@ -1050,29 +1118,29 @@ class CassandraStorage:
 
     def raw_extrinsic_metadata_get(
         self,
-        object_type: MetadataTargetType,
+        type: MetadataTargetType,
         id: Union[str, SWHID],
         authority: MetadataAuthority,
         after: Optional[datetime.datetime] = None,
         page_token: Optional[bytes] = None,
         limit: int = 1000,
-    ) -> Dict[str, Union[Optional[bytes], List[RawExtrinsicMetadata]]]:
-        if object_type == MetadataTargetType.ORIGIN:
+    ) -> PagedResult[RawExtrinsicMetadata]:
+        if type == MetadataTargetType.ORIGIN:
             if isinstance(id, SWHID):
                 raise StorageArgumentException(
-                    f"raw_extrinsic_metadata_get called with object_type='origin', "
+                    f"raw_extrinsic_metadata_get called with type='origin', "
                     f"but provided id is an SWHID: {id!r}"
                 )
         else:
             if not isinstance(id, SWHID):
                 raise StorageArgumentException(
-                    f"raw_extrinsic_metadata_get called with object_type!='origin', "
+                    f"raw_extrinsic_metadata_get called with type!='origin', "
                     f"but provided id is not an SWHID: {id!r}"
                 )
 
         if page_token is not None:
             (after_date, after_fetcher_name, after_fetcher_url) = msgpack_loads(
-                page_token
+                base64.b64decode(page_token)
             )
             if after and after_date < after:
                 raise StorageArgumentException(
@@ -1132,20 +1200,19 @@ class CassandraStorage:
             results.pop()
             assert len(results) == limit
             last_result = results[-1]
-            next_page_token: Optional[bytes] = msgpack_dumps(
-                (
-                    last_result.discovery_date,
-                    last_result.fetcher.name,
-                    last_result.fetcher.version,
+            next_page_token: Optional[str] = base64.b64encode(
+                msgpack_dumps(
+                    (
+                        last_result.discovery_date,
+                        last_result.fetcher.name,
+                        last_result.fetcher.version,
+                    )
                 )
-            )
+            ).decode()
         else:
             next_page_token = None
 
-        return {
-            "next_page_token": next_page_token,
-            "results": results,
-        }
+        return PagedResult(next_page_token=next_page_token, results=results,)
 
     def metadata_fetcher_add(self, fetchers: List[MetadataFetcher]) -> None:
         self.journal_writer.metadata_fetcher_add(fetchers)
