@@ -3,13 +3,14 @@
 # License: GNU General Public License version 3, or any later version
 # See top-level LICENSE file for more information
 
-import re
+import base64
 import bisect
 import collections
 import copy
 import datetime
 import itertools
 import random
+import re
 
 from collections import defaultdict
 from datetime import timedelta
@@ -23,6 +24,7 @@ from typing import (
     Iterator,
     List,
     Optional,
+    Set,
     Tuple,
     TypeVar,
     Union,
@@ -49,9 +51,11 @@ from swh.model.model import (
     MetadataFetcher,
     MetadataTargetType,
     RawExtrinsicMetadata,
+    Sha1,
+    Sha1Git,
 )
 from swh.model.hashutil import DEFAULT_ALGORITHMS, hash_to_bytes, hash_to_hex
-from swh.storage.interface import ListOrder, PagedResult
+from swh.storage.interface import ListOrder, PagedResult, VISIT_STATUSES
 from swh.storage.objstorage import ObjStorage
 from swh.storage.utils import now
 
@@ -136,7 +140,6 @@ class InMemoryStorage:
         self._releases = {}
         self._snapshots = {}
         self._origins = {}
-        self._origins_by_id = []
         self._origins_by_sha1 = {}
         self._origin_visits = {}
         self._origin_visit_statuses: Dict[Tuple[str, int], List[OriginVisitStatus]] = {}
@@ -174,7 +177,7 @@ class InMemoryStorage:
 
         self.objstorage = ObjStorage({"cls": "memory", "args": {}})
 
-    def check_config(self, *, check_write):
+    def check_config(self, *, check_write: bool) -> bool:
         return True
 
     def _content_add(self, contents: List[Content], with_data: bool) -> Dict:
@@ -225,10 +228,12 @@ class InMemoryStorage:
         content = [attr.evolve(c, ctime=now()) for c in content]
         return self._content_add(content, with_data=True)
 
-    def content_update(self, content, keys=[]):
-        self.journal_writer.content_update(content)
+    def content_update(
+        self, contents: List[Dict[str, Any]], keys: List[str] = []
+    ) -> None:
+        self.journal_writer.content_update(contents)
 
-        for cont_update in content:
+        for cont_update in contents:
             cont_update = cont_update.copy()
             sha1 = cont_update.pop("sha1")
             for old_key in self._content_indexes["sha1"][sha1]:
@@ -250,43 +255,17 @@ class InMemoryStorage:
     def content_add_metadata(self, content: List[Content]) -> Dict:
         return self._content_add(content, with_data=False)
 
-    def content_get(self, content):
-        # FIXME: Make this method support slicing the `data`.
-        if len(content) > BULK_BLOCK_CONTENT_LEN_MAX:
-            raise StorageArgumentException(
-                "Sending at most %s contents." % BULK_BLOCK_CONTENT_LEN_MAX
-            )
-        yield from self.objstorage.content_get(content)
-
-    def content_get_range(self, start, end, limit=1000):
-        if limit is None:
-            raise StorageArgumentException("limit should not be None")
-        sha1s = (
-            (sha1, content_key)
-            for sha1 in self._sorted_sha1s.iter_from(start)
-            for content_key in self._content_indexes["sha1"][sha1]
-        )
-        matched = []
-        next_content = None
-        for sha1, key in sha1s:
-            if sha1 > end:
-                break
-            if len(matched) >= limit:
-                next_content = sha1
-                break
-            matched.append(self._contents[key].to_dict())
-        return {
-            "contents": matched,
-            "next": next_content,
-        }
+    def content_get_data(self, content: Sha1) -> Optional[bytes]:
+        # FIXME: Make this method support slicing the `data`
+        return self.objstorage.content_get(content)
 
     def content_get_partition(
         self,
         partition_id: int,
         nb_partitions: int,
+        page_token: Optional[str] = None,
         limit: int = 1000,
-        page_token: str = None,
-    ):
+    ) -> PagedResult[Content]:
         if limit is None:
             raise StorageArgumentException("limit should not be None")
         (start, end) = get_partition_bounds_bytes(
@@ -296,35 +275,43 @@ class InMemoryStorage:
             start = hash_to_bytes(page_token)
         if end is None:
             end = b"\xff" * SHA1_SIZE
-        result = self.content_get_range(start, end, limit)
-        result2 = {
-            "contents": result["contents"],
-            "next_page_token": None,
-        }
-        if result["next"]:
-            result2["next_page_token"] = hash_to_hex(result["next"])
-        return result2
 
-    def content_get_metadata(self, contents: List[bytes]) -> Dict[bytes, List[Dict]]:
-        result: Dict = {sha1: [] for sha1 in contents}
+        next_page_token: Optional[str] = None
+        sha1s = (
+            (sha1, content_key)
+            for sha1 in self._sorted_sha1s.iter_from(start)
+            for content_key in self._content_indexes["sha1"][sha1]
+        )
+        contents: List[Content] = []
+        for counter, (sha1, key) in enumerate(sha1s):
+            if sha1 > end:
+                break
+            if counter >= limit:
+                next_page_token = hash_to_hex(sha1)
+                break
+            contents.append(self._contents[key])
+
+        assert len(contents) <= limit
+        return PagedResult(results=contents, next_page_token=next_page_token)
+
+    def content_get(self, contents: List[Sha1]) -> List[Optional[Content]]:
+        contents_by_sha1: Dict[Sha1, Optional[Content]] = {}
         for sha1 in contents:
             if sha1 in self._content_indexes["sha1"]:
                 objs = self._content_indexes["sha1"][sha1]
                 # only 1 element as content_add_metadata would have raised a
                 # hash collision otherwise
+                assert len(objs) == 1
                 for key in objs:
-                    d = self._contents[key].to_dict()
-                    del d["ctime"]
-                    if "data" in d:
-                        del d["data"]
-                    result[sha1].append(d)
-        return result
+                    content = attr.evolve(self._contents[key], data=None, ctime=None)
+                    contents_by_sha1[sha1] = content
+        return [contents_by_sha1.get(sha1) for sha1 in contents]
 
-    def content_find(self, content):
+    def content_find(self, content: Dict[str, Any]) -> List[Content]:
         if not set(content).intersection(DEFAULT_ALGORITHMS):
             raise StorageArgumentException(
-                "content keys must contain at least one of: %s"
-                % ", ".join(sorted(DEFAULT_ALGORITHMS))
+                "content keys must contain at least one "
+                f"of: {', '.join(sorted(DEFAULT_ALGORITHMS))}"
             )
         found = []
         for algo in DEFAULT_ALGORITHMS:
@@ -336,32 +323,37 @@ class InMemoryStorage:
             return []
 
         keys = list(set.intersection(*found))
-        return [self._contents[key].to_dict() for key in keys]
+        return [self._contents[key] for key in keys]
 
-    def content_missing(self, content, key_hash="sha1"):
-        for cont in content:
-            for (algo, hash_) in cont.items():
+    def content_missing(
+        self, contents: List[Dict[str, Any]], key_hash: str = "sha1"
+    ) -> Iterable[bytes]:
+        if key_hash not in DEFAULT_ALGORITHMS:
+            raise StorageArgumentException(
+                "key_hash should be one of {','.join(DEFAULT_ALGORITHMS)}"
+            )
+
+        for content in contents:
+            for (algo, hash_) in content.items():
                 if algo not in DEFAULT_ALGORITHMS:
                     continue
                 if hash_ not in self._content_indexes.get(algo, []):
-                    yield cont[key_hash]
+                    yield content[key_hash]
                     break
-            else:
-                for result in self.content_find(cont):
-                    if result["status"] == "missing":
-                        yield cont[key_hash]
 
-    def content_missing_per_sha1(self, contents):
+    def content_missing_per_sha1(self, contents: List[bytes]) -> Iterable[bytes]:
         for content in contents:
             if content not in self._content_indexes["sha1"]:
                 yield content
 
-    def content_missing_per_sha1_git(self, contents):
+    def content_missing_per_sha1_git(
+        self, contents: List[Sha1Git]
+    ) -> Iterable[Sha1Git]:
         for content in contents:
             if content not in self._content_indexes["sha1_git"]:
                 yield content
 
-    def content_get_random(self):
+    def content_get_random(self) -> Sha1Git:
         return random.choice(list(self._content_indexes["sha1_git"]))
 
     def _skipped_content_add(self, contents: List[SkippedContent]) -> Dict:
@@ -386,7 +378,9 @@ class InMemoryStorage:
         content = [attr.evolve(c, ctime=now()) for c in content]
         return self._skipped_content_add(content)
 
-    def skipped_content_missing(self, contents):
+    def skipped_content_missing(
+        self, contents: List[Dict[str, Any]]
+    ) -> Iterable[Dict[str, Any]]:
         for content in contents:
             matches = list(self._skipped_contents.values())
             for (algorithm, key) in self._content_key(content):
@@ -412,12 +406,12 @@ class InMemoryStorage:
 
         return {"directory:add": count}
 
-    def directory_missing(self, directories):
+    def directory_missing(self, directories: List[Sha1Git]) -> Iterable[Sha1Git]:
         for id in directories:
             if id not in self._directories:
                 yield id
 
-    def _join_dentry_to_content(self, dentry):
+    def _join_dentry_to_content(self, dentry: Dict[str, Any]) -> Dict[str, Any]:
         keys = (
             "status",
             "sha1",
@@ -429,11 +423,11 @@ class InMemoryStorage:
         ret.update(dentry)
         if ret["type"] == "file":
             # TODO: Make it able to handle more than one content
-            content = self.content_find({"sha1_git": ret["target"]})
-            if content:
-                content = content[0]
+            contents = self.content_find({"sha1_git": ret["target"]})
+            if contents:
+                content = contents[0]
                 for key in keys:
-                    ret[key] = content[key]
+                    ret[key] = getattr(content, key)
         return ret
 
     def _directory_ls(self, directory_id, recursive, prefix=b""):
@@ -448,25 +442,29 @@ class InMemoryStorage:
                         ret["target"], True, prefix + ret["name"] + b"/"
                     )
 
-    def directory_ls(self, directory, recursive=False):
+    def directory_ls(
+        self, directory: Sha1Git, recursive: bool = False
+    ) -> Iterable[Dict[str, Any]]:
         yield from self._directory_ls(directory, recursive)
 
-    def directory_entry_get_by_path(self, directory, paths):
+    def directory_entry_get_by_path(
+        self, directory: Sha1Git, paths: List[bytes]
+    ) -> Optional[Dict[str, Any]]:
         return self._directory_entry_get_by_path(directory, paths, b"")
 
-    def directory_get_random(self):
-        if not self._directories:
-            return None
+    def directory_get_random(self) -> Sha1Git:
         return random.choice(list(self._directories))
 
-    def _directory_entry_get_by_path(self, directory, paths, prefix):
+    def _directory_entry_get_by_path(
+        self, directory: Sha1Git, paths: List[bytes], prefix: bytes
+    ) -> Optional[Dict[str, Any]]:
         if not paths:
-            return
+            return None
 
         contents = list(self.directory_ls(directory))
 
         if not contents:
-            return
+            return None
 
         def _get_entry(entries, name):
             for entry in entries:
@@ -481,7 +479,7 @@ class InMemoryStorage:
             return first_item
 
         if not first_item or first_item["type"] != "dir":
-            return
+            return None
 
         return self._directory_entry_get_by_path(
             first_item["target"], paths[1:], prefix + paths[0] + b"/"
@@ -504,19 +502,23 @@ class InMemoryStorage:
 
         return {"revision:add": count}
 
-    def revision_missing(self, revisions):
+    def revision_missing(self, revisions: List[Sha1Git]) -> Iterable[Sha1Git]:
         for id in revisions:
             if id not in self._revisions:
                 yield id
 
-    def revision_get(self, revisions):
+    def revision_get(
+        self, revisions: List[Sha1Git]
+    ) -> Iterable[Optional[Dict[str, Any]]]:
         for id in revisions:
             if id in self._revisions:
                 yield self._revisions.get(id).to_dict()
             else:
                 yield None
 
-    def _get_parent_revs(self, rev_id, seen, limit):
+    def _get_parent_revs(
+        self, rev_id: Sha1Git, seen: Set[Sha1Git], limit: Optional[int]
+    ) -> Iterable[Dict[str, Any]]:
         if limit and len(seen) >= limit:
             return
         if rev_id in seen or rev_id not in self._revisions:
@@ -526,17 +528,22 @@ class InMemoryStorage:
         for parent in self._revisions[rev_id].parents:
             yield from self._get_parent_revs(parent, seen, limit)
 
-    def revision_log(self, revisions, limit=None):
-        seen = set()
+    def revision_log(
+        self, revisions: List[Sha1Git], limit: Optional[int] = None
+    ) -> Iterable[Optional[Dict[str, Any]]]:
+        seen: Set[Sha1Git] = set()
         for rev_id in revisions:
             yield from self._get_parent_revs(rev_id, seen, limit)
 
-    def revision_shortlog(self, revisions, limit=None):
+    def revision_shortlog(
+        self, revisions: List[Sha1Git], limit: Optional[int] = None
+    ) -> Iterable[Optional[Tuple[Sha1Git, Tuple[Sha1Git, ...]]]]:
         yield from (
-            (rev["id"], rev["parents"]) for rev in self.revision_log(revisions, limit)
+            (rev["id"], rev["parents"]) if rev else None
+            for rev in self.revision_log(revisions, limit)
         )
 
-    def revision_get_random(self):
+    def revision_get_random(self) -> Sha1Git:
         return random.choice(list(self._revisions))
 
     def release_add(self, releases: List[Release]) -> Dict:
@@ -554,17 +561,19 @@ class InMemoryStorage:
 
         return {"release:add": len(to_add)}
 
-    def release_missing(self, releases):
+    def release_missing(self, releases: List[Sha1Git]) -> Iterable[Sha1Git]:
         yield from (rel for rel in releases if rel not in self._releases)
 
-    def release_get(self, releases):
+    def release_get(
+        self, releases: List[Sha1Git]
+    ) -> Iterable[Optional[Dict[str, Any]]]:
         for rel_id in releases:
             if rel_id in self._releases:
                 yield self._releases[rel_id].to_dict()
             else:
                 yield None
 
-    def release_get_random(self):
+    def release_get_random(self) -> Sha1Git:
         return random.choice(list(self._releases))
 
     def snapshot_add(self, snapshots: List[Snapshot]) -> Dict:
@@ -578,32 +587,34 @@ class InMemoryStorage:
 
         return {"snapshot:add": count}
 
-    def snapshot_missing(self, snapshots):
+    def snapshot_missing(self, snapshots: List[Sha1Git]) -> Iterable[Sha1Git]:
         for id in snapshots:
             if id not in self._snapshots:
                 yield id
 
-    def snapshot_get(self, snapshot_id):
+    def snapshot_get(self, snapshot_id: Sha1Git) -> Optional[Dict[str, Any]]:
         return self.snapshot_get_branches(snapshot_id)
 
-    def snapshot_get_by_origin_visit(self, origin, visit):
+    def snapshot_get_by_origin_visit(
+        self, origin: str, visit: int
+    ) -> Optional[Dict[str, Any]]:
         origin_url = self._get_origin_url(origin)
         if not origin_url:
-            return
+            return None
 
         if origin_url not in self._origins or visit > len(
             self._origin_visits[origin_url]
         ):
             return None
 
-        visit = self._origin_visit_get_updated(origin_url, visit)
-        snapshot_id = visit["snapshot"]
+        visit_d = self._origin_visit_get_updated(origin_url, visit)
+        snapshot_id = visit_d["snapshot"]
         if snapshot_id:
             return self.snapshot_get(snapshot_id)
         else:
             return None
 
-    def snapshot_count_branches(self, snapshot_id):
+    def snapshot_count_branches(self, snapshot_id: Sha1Git) -> Optional[Dict[str, int]]:
         snapshot = self._snapshots[snapshot_id]
         return collections.Counter(
             branch.target_type.value if branch else None
@@ -611,8 +622,12 @@ class InMemoryStorage:
         )
 
     def snapshot_get_branches(
-        self, snapshot_id, branches_from=b"", branches_count=1000, target_types=None
-    ):
+        self,
+        snapshot_id: Sha1Git,
+        branches_from: bytes = b"",
+        branches_count: int = 1000,
+        target_types: Optional[List[str]] = None,
+    ) -> Optional[Dict[str, Any]]:
         snapshot = self._snapshots.get(snapshot_id)
         if snapshot is None:
             return None
@@ -621,7 +636,7 @@ class InMemoryStorage:
         from_index = bisect.bisect_left(sorted_branch_names, branches_from)
         if target_types:
             next_branch = None
-            branches = {}
+            branches: Dict = {}
             for (branch_name, branch) in sorted_branches:
                 if branch_name in sorted_branch_names[from_index:]:
                     if branch and branch.target_type.value in target_types:
@@ -655,11 +670,11 @@ class InMemoryStorage:
             "next_branch": next_branch,
         }
 
-    def snapshot_get_random(self):
+    def snapshot_get_random(self) -> Sha1Git:
         return random.choice(list(self._snapshots))
 
-    def object_find_by_sha1_git(self, ids):
-        ret = {}
+    def object_find_by_sha1_git(self, ids: List[Sha1Git]) -> Dict[Sha1Git, List[Dict]]:
+        ret: Dict[Sha1Git, List[Dict]] = {}
         for id_ in ids:
             objs = self._objects.get(id_, [])
             ret[id_] = [{"sha1_git": id_, "type": obj[0],} for obj in objs]
@@ -677,19 +692,9 @@ class InMemoryStorage:
     def origin_get(self, origins: List[str]) -> Iterable[Optional[Origin]]:
         return [self.origin_get_one(origin_url) for origin_url in origins]
 
-    def origin_get_by_sha1(self, sha1s):
+    def origin_get_by_sha1(self, sha1s: List[bytes]) -> List[Optional[Dict[str, Any]]]:
         return [self._convert_origin(self._origins_by_sha1.get(sha1)) for sha1 in sha1s]
 
-    def origin_get_range(self, origin_from=1, origin_count=100):
-        origin_from = max(origin_from, 1)
-        if origin_from <= len(self._origins_by_id):
-            max_idx = origin_from + origin_count - 1
-            if max_idx > len(self._origins_by_id):
-                max_idx = len(self._origins_by_id)
-            for idx in range(origin_from - 1, max_idx):
-                origin = self._convert_origin(self._origins[self._origins_by_id[idx]])
-                yield {"id": idx + 1, **origin}
-
     def origin_list(
         self, page_token: Optional[str] = None, limit: int = 100
     ) -> PagedResult[Origin]:
@@ -711,20 +716,29 @@ class InMemoryStorage:
         return PagedResult(results=origins, next_page_token=next_page_token)
 
     def origin_search(
-        self, url_pattern, offset=0, limit=50, regexp=False, with_visit=False
-    ):
-        origins = map(self._convert_origin, self._origins.values())
+        self,
+        url_pattern: str,
+        page_token: Optional[str] = None,
+        limit: int = 50,
+        regexp: bool = False,
+        with_visit: bool = False,
+    ) -> PagedResult[Origin]:
+        next_page_token = None
+        offset = int(page_token) if page_token else 0
+
+        origins = self._origins.values()
         if regexp:
             pat = re.compile(url_pattern)
-            origins = [orig for orig in origins if pat.search(orig["url"])]
+            origins = [orig for orig in origins if pat.search(orig.url)]
         else:
-            origins = [orig for orig in origins if url_pattern in orig["url"]]
+            origins = [orig for orig in origins if url_pattern in orig.url]
+
         if with_visit:
             filtered_origins = []
             for orig in origins:
                 visits = (
                     self._origin_visit_get_updated(ov.origin, ov.visit)
-                    for ov in self._origin_visits[orig["url"]]
+                    for ov in self._origin_visits[orig.url]
                 )
                 for ov in visits:
                     snapshot = ov["snapshot"]
@@ -734,17 +748,25 @@ class InMemoryStorage:
         else:
             filtered_origins = origins
 
-        return filtered_origins[offset : offset + limit]
+        # Take one more origin so we can reuse it as the next page token if any
+        origins = filtered_origins[offset : offset + limit + 1]
+        if len(origins) > limit:
+            # next offset
+            next_page_token = str(offset + limit)
+            # excluding that origin from the result to respect the limit size
+            origins = origins[:limit]
 
-    def origin_count(self, url_pattern, regexp=False, with_visit=False):
-        return len(
-            self.origin_search(
-                url_pattern,
-                regexp=regexp,
-                with_visit=with_visit,
-                limit=len(self._origins),
-            )
+        assert len(origins) <= limit
+        return PagedResult(results=origins, next_page_token=next_page_token)
+
+    def origin_count(
+        self, url_pattern: str, regexp: bool = False, with_visit: bool = False
+    ) -> int:
+        actual_page = self.origin_search(
+            url_pattern, regexp=regexp, with_visit=with_visit, limit=len(self._origins),
         )
+        assert actual_page.next_page_token is None
+        return len(actual_page.results)
 
     def origin_add(self, origins: List[Origin]) -> Dict[str, int]:
         added = 0
@@ -758,12 +780,6 @@ class InMemoryStorage:
     def origin_add_one(self, origin: Origin) -> str:
         if origin.url not in self._origins:
             self.journal_writer.origin_add([origin])
-            # generate an origin_id because it is needed by origin_get_range.
-            # TODO: remove this when we remove origin_get_range
-            origin_id = len(self._origins) + 1
-            self._origins_by_id.append(origin.url)
-            assert len(self._origins_by_id) == origin_id
-
             self._origins[origin.url] = origin
             self._origins_by_sha1[origin_url_to_sha1(origin.url)] = origin
             self._origin_visits[origin.url] = []
@@ -921,6 +937,12 @@ class InMemoryStorage:
         allowed_statuses: Optional[List[str]] = None,
         require_snapshot: bool = False,
     ) -> Optional[OriginVisit]:
+        if allowed_statuses and not set(allowed_statuses).intersection(VISIT_STATUSES):
+            raise StorageArgumentException(
+                f"Unknown allowed statuses {','.join(allowed_statuses)}, only "
+                f"{','.join(VISIT_STATUSES)} authorized"
+            )
+
         ori = self._origins.get(origin)
         if not ori:
             return None
@@ -991,6 +1013,12 @@ class InMemoryStorage:
         allowed_statuses: Optional[List[str]] = None,
         require_snapshot: bool = False,
     ) -> Optional[OriginVisitStatus]:
+        if allowed_statuses and not set(allowed_statuses).intersection(VISIT_STATUSES):
+            raise StorageArgumentException(
+                f"Unknown allowed statuses {','.join(allowed_statuses)}, only "
+                f"{','.join(VISIT_STATUSES)} authorized"
+            )
+
         ori = self._origins.get(origin_url)
         if not ori:
             return None
@@ -1093,45 +1121,45 @@ class InMemoryStorage:
 
     def raw_extrinsic_metadata_get(
         self,
-        object_type: MetadataTargetType,
+        type: MetadataTargetType,
         id: Union[str, SWHID],
         authority: MetadataAuthority,
         after: Optional[datetime.datetime] = None,
         page_token: Optional[bytes] = None,
         limit: int = 1000,
-    ) -> Dict[str, Union[Optional[bytes], List[RawExtrinsicMetadata]]]:
+    ) -> PagedResult[RawExtrinsicMetadata]:
         authority_key = self._metadata_authority_key(authority)
 
-        if object_type == MetadataTargetType.ORIGIN:
+        if type == MetadataTargetType.ORIGIN:
             if isinstance(id, SWHID):
                 raise StorageArgumentException(
-                    f"raw_extrinsic_metadata_get called with object_type='origin', "
+                    f"raw_extrinsic_metadata_get called with type='origin', "
                     f"but provided id is an SWHID: {id!r}"
                 )
         else:
             if not isinstance(id, SWHID):
                 raise StorageArgumentException(
-                    f"raw_extrinsic_metadata_get called with object_type!='origin', "
+                    f"raw_extrinsic_metadata_get called with type!='origin', "
                     f"but provided id is not an SWHID: {id!r}"
                 )
 
         if page_token is not None:
-            (after_time, after_fetcher) = msgpack_loads(page_token)
+            (after_time, after_fetcher) = msgpack_loads(base64.b64decode(page_token))
             after_fetcher = tuple(after_fetcher)
             if after is not None and after > after_time:
                 raise StorageArgumentException(
                     "page_token is inconsistent with the value of 'after'."
                 )
-            entries = self._raw_extrinsic_metadata[object_type][id][
-                authority_key
-            ].iter_after((after_time, after_fetcher))
+            entries = self._raw_extrinsic_metadata[type][id][authority_key].iter_after(
+                (after_time, after_fetcher)
+            )
         elif after is not None:
-            entries = self._raw_extrinsic_metadata[object_type][id][
-                authority_key
-            ].iter_from((after,))
+            entries = self._raw_extrinsic_metadata[type][id][authority_key].iter_from(
+                (after,)
+            )
             entries = (entry for entry in entries if entry.discovery_date > after)
         else:
-            entries = iter(self._raw_extrinsic_metadata[object_type][id][authority_key])
+            entries = iter(self._raw_extrinsic_metadata[type][id][authority_key])
 
         if limit:
             entries = itertools.islice(entries, 0, limit + 1)
@@ -1158,19 +1186,18 @@ class InMemoryStorage:
             results.pop()
             assert len(results) == limit
             last_result = results[-1]
-            next_page_token: Optional[bytes] = msgpack_dumps(
-                (
-                    last_result.discovery_date,
-                    self._metadata_fetcher_key(last_result.fetcher),
+            next_page_token: Optional[str] = base64.b64encode(
+                msgpack_dumps(
+                    (
+                        last_result.discovery_date,
+                        self._metadata_fetcher_key(last_result.fetcher),
+                    )
                 )
-            )
+            ).decode()
         else:
             next_page_token = None
 
-        return {
-            "next_page_token": next_page_token,
-            "results": results,
-        }
+        return PagedResult(next_page_token=next_page_token, results=results,)
 
     def metadata_fetcher_add(self, fetchers: List[MetadataFetcher]) -> None:
         self.journal_writer.metadata_fetcher_add(fetchers)
