@@ -6,7 +6,7 @@
 import datetime
 
 from enum import Enum
-from typing import Dict, Iterable, List, Optional, Tuple, TypeVar, Union
+from typing import Any, Dict, Iterable, List, Optional, Tuple, TypeVar, Union
 
 
 from swh.core.api import remote_api_endpoint
@@ -27,6 +27,8 @@ from swh.model.model import (
     MetadataFetcher,
     MetadataTargetType,
     RawExtrinsicMetadata,
+    Sha1,
+    Sha1Git,
 )
 
 
@@ -41,6 +43,10 @@ TResult = TypeVar("TResult")
 PagedResult = CorePagedResult[TResult, str]
 
 
+# TODO: Make it an enum (too much impact)
+VISIT_STATUSES = ["created", "ongoing", "full", "partial"]
+
+
 def deprecated(f):
     f.deprecated_endpoint = True
     return f
@@ -48,7 +54,7 @@ def deprecated(f):
 
 class StorageInterface:
     @remote_api_endpoint("check_config")
-    def check_config(self, *, check_write):
+    def check_config(self, *, check_write: bool) -> bool:
         """Check that the storage is configured and ready to go."""
         ...
 
@@ -88,12 +94,14 @@ class StorageInterface:
         ...
 
     @remote_api_endpoint("content/update")
-    def content_update(self, content, keys=[]):
+    def content_update(
+        self, contents: List[Dict[str, Any]], keys: List[str] = []
+    ) -> None:
         """Update content blobs to the storage. Does nothing for unknown
         contents or skipped ones.
 
         Args:
-            content (iterable): iterable of dictionaries representing
+            content: iterable of dictionaries representing
                 individual pieces of content to update. Each dictionary has the
                 following keys:
 
@@ -139,52 +147,14 @@ class StorageInterface:
         ...
 
     @remote_api_endpoint("content/data")
-    def content_get(self, content):
-        """Retrieve in bulk contents and their data.
-
-        This generator yields exactly as many items than provided sha1
-        identifiers, but callers should not assume this will always be true.
-
-        It may also yield `None` values in case an object was not found.
+    def content_get_data(self, content: Sha1) -> Optional[bytes]:
+        """Given a content identifier, returns its associated data if any.
 
         Args:
-            content: iterables of sha1
-
-        Yields:
-            Dict[str, bytes]: Generates streams of contents as dict with their
-                raw data:
-
-                - sha1 (bytes): content id
-                - data (bytes): content's raw data
-
-        Raises:
-            ValueError in case of too much contents are required.
-            cf. BULK_BLOCK_CONTENT_LEN_MAX
-
-        """
-        ...
-
-    @deprecated
-    @remote_api_endpoint("content/range")
-    def content_get_range(self, start, end, limit=1000):
-        """Retrieve contents within range [start, end] bound by limit.
-
-        Note that this function may return more than one blob per hash. The
-        limit is enforced with multiplicity (ie. two blobs with the same hash
-        will count twice toward the limit).
-
-        Args:
-            **start** (bytes): Starting identifier range (expected smaller
-                           than end)
-            **end** (bytes): Ending identifier range (expected larger
-                             than start)
-            **limit** (int): Limit result (default to 1000)
+            content: sha1 identifier
 
         Returns:
-            a dict with keys:
-            - contents [dict]: iterable of contents in between the range.
-            - next (bytes): There remains content in the range
-              starting from this next sha1
+             raw content data (bytes)
 
         """
         ...
@@ -194,9 +164,9 @@ class StorageInterface:
         self,
         partition_id: int,
         nb_partitions: int,
+        page_token: Optional[str] = None,
         limit: int = 1000,
-        page_token: str = None,
-    ):
+    ) -> PagedResult[Content]:
         """Splits contents into nb_partitions, and returns one of these based on
         partition_id (which must be in [0, nb_partitions-1])
 
@@ -204,77 +174,73 @@ class StorageInterface:
         result order.
 
         Args:
-            partition_id (int): index of the partition to fetch
-            nb_partitions (int): total number of partitions to split into
-            limit (int): Limit result (default to 1000)
-            page_token (Optional[str]): opaque token used for pagination.
+            partition_id: index of the partition to fetch
+            nb_partitions: total number of partitions to split into
+            page_token: opaque token used for pagination.
+            limit: Limit result (default to 1000)
 
         Returns:
-            a dict with keys:
-              - contents (List[dict]): iterable of contents in the partition.
-              - **next_page_token** (Optional[str]): opaque token to be used as
-                `page_token` for retrieving the next page. if absent, there is
-                no more pages to gather.
+            PagedResult of Content model objects within the partition. If
+            next_page_token is None, there is no longer data to retrieve.
+
         """
         ...
 
     @remote_api_endpoint("content/metadata")
-    def content_get_metadata(self, contents: List[bytes]) -> Dict[bytes, List[Dict]]:
+    def content_get(self, contents: List[Sha1]) -> List[Optional[Content]]:
         """Retrieve content metadata in bulk
 
         Args:
-            content: iterable of content identifiers (sha1)
+            content: List of content identifiers
 
         Returns:
-            a dict with keys the content's sha1 and the associated value
-            either the existing content's metadata or None if the content does
-            not exist.
+            List of contents model objects when they exist, None otherwise.
 
         """
         ...
 
     @remote_api_endpoint("content/missing")
-    def content_missing(self, content, key_hash="sha1"):
+    def content_missing(
+        self, contents: List[Dict[str, Any]], key_hash: str = "sha1"
+    ) -> Iterable[bytes]:
         """List content missing from storage
 
         Args:
-            content ([dict]): iterable of dictionaries whose keys are
-                              either 'length' or an item of
-                              :data:`swh.model.hashutil.ALGORITHMS`;
-                              mapped to the corresponding checksum
-                              (or length).
-
-            key_hash (str): name of the column to use as hash id
-                            result (default: 'sha1')
-
-        Returns:
-            iterable ([bytes]): missing content ids (as per the
-            key_hash column)
+            content: iterable of dictionaries whose keys are either 'length' or an item
+                of :data:`swh.model.hashutil.ALGORITHMS`; mapped to the
+                corresponding checksum (or length).
+            key_hash: name of the column to use as hash id result (default: 'sha1')
 
         Raises:
+            StorageArgumentException when key_hash is unknown.
             TODO: an exception when we get a hash collision.
 
+        Returns:
+            iterable of missing content ids (as per the `key_hash` column)
+
         """
         ...
 
     @remote_api_endpoint("content/missing/sha1")
-    def content_missing_per_sha1(self, contents):
+    def content_missing_per_sha1(self, contents: List[bytes]) -> Iterable[bytes]:
         """List content missing from storage based only on sha1.
 
         Args:
             contents: List of sha1 to check for absence.
 
-        Returns:
-            iterable: missing ids
-
         Raises:
             TODO: an exception when we get a hash collision.
 
+        Returns:
+            Iterable of missing content ids (sha1)
+
         """
         ...
 
     @remote_api_endpoint("content/missing/sha1_git")
-    def content_missing_per_sha1_git(self, contents):
+    def content_missing_per_sha1_git(
+        self, contents: List[Sha1Git]
+    ) -> Iterable[Sha1Git]:
         """List content missing from storage based only on sha1_git.
 
         Args:
@@ -282,11 +248,12 @@ class StorageInterface:
 
         Yields:
             missing contents sha1_git
+
         """
         ...
 
     @remote_api_endpoint("content/present")
-    def content_find(self, content):
+    def content_find(self, content: Dict[str, Any]) -> List[Content]:
         """Find a content hash in db.
 
         Args:
@@ -294,19 +261,19 @@ class StorageInterface:
                 checksum algorithm names (see swh.model.hashutil.ALGORITHMS) to
                 checksum values
 
-        Returns:
-            a triplet (sha1, sha1_git, sha256) if the content exist
-            or None otherwise.
-
         Raises:
             ValueError: in case the key of the dictionary is not sha1, sha1_git
                 nor sha256.
 
+        Returns:
+            an iterable of Content objects matching the search criteria if the
+            content exist. Empty iterable otherwise.
+
         """
         ...
 
     @remote_api_endpoint("content/get_random")
-    def content_get_random(self):
+    def content_get_random(self) -> Sha1Git:
         """Finds a random content id.
 
         Returns:
@@ -352,15 +319,17 @@ class StorageInterface:
         ...
 
     @remote_api_endpoint("content/skipped/missing")
-    def skipped_content_missing(self, contents):
-        """List skipped_content missing from storage
+    def skipped_content_missing(
+        self, contents: List[Dict[str, Any]]
+    ) -> Iterable[Dict[str, Any]]:
+        """List skipped contents missing from storage.
 
         Args:
-            content: iterable of dictionaries containing the data for each
+            contents: iterable of dictionaries containing the data for each
                 checksum algorithm.
 
         Returns:
-            iterable: missing signatures
+            Iterable of missing skipped contents as dict
 
         """
         ...
@@ -394,11 +363,11 @@ class StorageInterface:
         ...
 
     @remote_api_endpoint("directory/missing")
-    def directory_missing(self, directories):
-        """List directories missing from storage
+    def directory_missing(self, directories: List[Sha1Git]) -> Iterable[Sha1Git]:
+        """List directories missing from storage.
 
         Args:
-            directories (iterable): an iterable of directory ids
+            directories: list of directory ids
 
         Yields:
             missing directory ids
@@ -407,39 +376,43 @@ class StorageInterface:
         ...
 
     @remote_api_endpoint("directory/ls")
-    def directory_ls(self, directory, recursive=False):
-        """Get entries for one directory.
-
-        Args:
-            - directory: the directory to list entries from.
-            - recursive: if flag on, this list recursively from this directory.
-
-        Returns:
-            List of entries for such directory.
+    def directory_ls(
+        self, directory: Sha1Git, recursive: bool = False
+    ) -> Iterable[Dict[str, Any]]:
+        """List entries for one directory.
 
         If `recursive=True`, names in the path of a dir/file not at the
         root are concatenated with a slash (`/`).
 
+        Args:
+            directory: the directory to list entries from.
+            recursive: if flag on, this list recursively from this directory.
+
+        Yields:
+            directory entries for such directory.
+
         """
         ...
 
     @remote_api_endpoint("directory/path")
-    def directory_entry_get_by_path(self, directory, paths):
+    def directory_entry_get_by_path(
+        self, directory: Sha1Git, paths: List[bytes]
+    ) -> Optional[Dict[str, Any]]:
         """Get the directory entry (either file or dir) from directory with path.
 
         Args:
-            - directory: sha1 of the top level directory
-            - paths: path to lookup from the top level directory. From left
+            directory: directory id
+            paths: path to lookup from the top level directory. From left
               (top) to right (bottom).
 
         Returns:
-            The corresponding directory entry if found, None otherwise.
+            The corresponding directory entry as dict if found, None otherwise.
 
         """
         ...
 
     @remote_api_endpoint("directory/get_random")
-    def directory_get_random(self):
+    def directory_get_random(self) -> Sha1Git:
         """Finds a random directory id.
 
         Returns:
@@ -488,11 +461,11 @@ class StorageInterface:
         ...
 
     @remote_api_endpoint("revision/missing")
-    def revision_missing(self, revisions):
+    def revision_missing(self, revisions: List[Sha1Git]) -> Iterable[Sha1Git]:
         """List revisions missing from storage
 
         Args:
-            revisions (iterable): revision ids
+            revisions: revision ids
 
         Yields:
             missing revision ids
@@ -501,35 +474,40 @@ class StorageInterface:
         ...
 
     @remote_api_endpoint("revision")
-    def revision_get(self, revisions):
-        """Get all revisions from storage
+    def revision_get(
+        self, revisions: List[Sha1Git]
+    ) -> Iterable[Optional[Dict[str, Any]]]:
+        """Get revisions from storage
 
         Args:
-            revisions: an iterable of revision ids
+            revisions: revision ids
 
-        Returns:
-            iterable: an iterable of revisions as dictionaries (or None if the
-                revision doesn't exist)
+        Yields:
+            revisions as dictionaries (or None if the revision doesn't exist)
 
         """
         ...
 
     @remote_api_endpoint("revision/log")
-    def revision_log(self, revisions, limit=None):
+    def revision_log(
+        self, revisions: List[Sha1Git], limit: Optional[int] = None
+    ) -> Iterable[Optional[Dict[str, Any]]]:
         """Fetch revision entry from the given root revisions.
 
         Args:
-            revisions: array of root revision to lookup
+            revisions: array of root revisions to lookup
             limit: limitation on the output result. Default to None.
 
         Yields:
-            List of revision log from such revisions root.
+            revision entries log from the given root root revisions
 
         """
         ...
 
     @remote_api_endpoint("revision/shortlog")
-    def revision_shortlog(self, revisions, limit=None):
+    def revision_shortlog(
+        self, revisions: List[Sha1Git], limit: Optional[int] = None
+    ) -> Iterable[Optional[Tuple[Sha1Git, Tuple[Sha1Git, ...]]]]:
         """Fetch the shortlog for the given revisions
 
         Args:
@@ -537,13 +515,13 @@ class StorageInterface:
             limit: depth limitation for the output
 
         Yields:
-            a list of (id, parents) tuples.
+            a list of (id, parents) tuples
 
         """
         ...
 
     @remote_api_endpoint("revision/get_random")
-    def revision_get_random(self):
+    def revision_get_random(self) -> Sha1Git:
         """Finds a random revision id.
 
         Returns:
@@ -581,20 +559,22 @@ class StorageInterface:
         ...
 
     @remote_api_endpoint("release/missing")
-    def release_missing(self, releases):
-        """List releases missing from storage
+    def release_missing(self, releases: List[Sha1Git]) -> Iterable[Sha1Git]:
+        """List missing release ids from storage
 
         Args:
-            releases: an iterable of release ids
+            releases: release ids
 
-        Returns:
+        Yields:
             a list of missing release ids
 
         """
         ...
 
     @remote_api_endpoint("release")
-    def release_get(self, releases):
+    def release_get(
+        self, releases: List[Sha1Git]
+    ) -> Iterable[Optional[Dict[str, Any]]]:
         """Given a list of sha1, return the releases's information
 
         Args:
@@ -608,7 +588,7 @@ class StorageInterface:
         ...
 
     @remote_api_endpoint("release/get_random")
-    def release_get_random(self):
+    def release_get_random(self) -> Sha1Git:
         """Finds a random release id.
 
         Returns:
@@ -650,11 +630,11 @@ class StorageInterface:
         ...
 
     @remote_api_endpoint("snapshot/missing")
-    def snapshot_missing(self, snapshots):
+    def snapshot_missing(self, snapshots: List[Sha1Git]) -> Iterable[Sha1Git]:
         """List snapshots missing from storage
 
         Args:
-            snapshots (iterable): an iterable of snapshot ids
+            snapshots: snapshot ids
 
         Yields:
             missing snapshot ids
@@ -663,7 +643,7 @@ class StorageInterface:
         ...
 
     @remote_api_endpoint("snapshot")
-    def snapshot_get(self, snapshot_id):
+    def snapshot_get(self, snapshot_id: Sha1Git) -> Optional[Dict[str, Any]]:
         """Get the content, possibly partial, of a snapshot with the given id
 
         The branches of the snapshot are iterated in the lexicographical
@@ -675,7 +655,8 @@ class StorageInterface:
             should be used instead.
 
         Args:
-            snapshot_id (bytes): identifier of the snapshot
+            snapshot_id: snapshot identifier
+
         Returns:
             dict: a dict with three keys:
                 * **id**: identifier of the snapshot
@@ -688,7 +669,9 @@ class StorageInterface:
         ...
 
     @remote_api_endpoint("snapshot/by_origin_visit")
-    def snapshot_get_by_origin_visit(self, origin, visit):
+    def snapshot_get_by_origin_visit(
+        self, origin: str, visit: int
+    ) -> Optional[Dict[str, Any]]:
         """Get the content, possibly partial, of a snapshot for the given origin visit
 
         The branches of the snapshot are iterated in the lexicographical
@@ -700,8 +683,9 @@ class StorageInterface:
             should be used instead.
 
         Args:
-            origin (int): the origin identifier
-            visit (int): the visit identifier
+            origin: origin identifier (url)
+            visit: the visit identifier
+
         Returns:
             dict: None if the snapshot does not exist;
               a dict with three keys otherwise:
@@ -716,37 +700,43 @@ class StorageInterface:
         ...
 
     @remote_api_endpoint("snapshot/count_branches")
-    def snapshot_count_branches(self, snapshot_id):
+    def snapshot_count_branches(self, snapshot_id: Sha1Git) -> Optional[Dict[str, int]]:
         """Count the number of branches in the snapshot with the given id
 
         Args:
-            snapshot_id (bytes): identifier of the snapshot
+            snapshot_id: snapshot identifier
 
         Returns:
-            dict: A dict whose keys are the target types of branches and
-            values their corresponding amount
+            A dict whose keys are the target types of branches and values their
+            corresponding amount
+
         """
         ...
 
     @remote_api_endpoint("snapshot/get_branches")
     def snapshot_get_branches(
-        self, snapshot_id, branches_from=b"", branches_count=1000, target_types=None
-    ):
+        self,
+        snapshot_id: Sha1Git,
+        branches_from: bytes = b"",
+        branches_count: int = 1000,
+        target_types: Optional[List[str]] = None,
+    ) -> Optional[Dict[str, Any]]:
         """Get the content, possibly partial, of a snapshot with the given id
 
         The branches of the snapshot are iterated in the lexicographical
         order of their names.
 
         Args:
-            snapshot_id (bytes): identifier of the snapshot
-            branches_from (bytes): optional parameter used to skip branches
+            snapshot_id: identifier of the snapshot
+            branches_from: optional parameter used to skip branches
                 whose name is lesser than it before returning them
-            branches_count (int): optional parameter used to restrain
+            branches_count: optional parameter used to restrain
                 the amount of returned branches
-            target_types (list): optional parameter used to filter the
+            target_types: optional parameter used to filter the
                 target types of branch to return (possible values that can be
                 contained in that list are `'content', 'directory',
                 'revision', 'release', 'snapshot', 'alias'`)
+
         Returns:
             dict: None if the snapshot does not exist;
               a dict with three keys otherwise:
@@ -760,7 +750,7 @@ class StorageInterface:
         ...
 
     @remote_api_endpoint("snapshot/get_random")
-    def snapshot_get_random(self):
+    def snapshot_get_random(self) -> Sha1Git:
         """Finds a random snapshot id.
 
         Returns:
@@ -883,6 +873,10 @@ class StorageInterface:
             require_snapshot: If True, only a visit with a snapshot
                 will be returned.
 
+        Raises:
+            StorageArgumentException if values for the allowed_statuses parameters
+            are unknown
+
         Returns:
             OriginVisit matching the criteria if found, None otherwise. Note that as
             OriginVisit no longer held reference on the visit status or snapshot, you
@@ -937,6 +931,10 @@ class StorageInterface:
             require_snapshot: If True, only a visit with a snapshot
                 will be returned.
 
+        Raises:
+            StorageArgumentException if values for the allowed_statuses parameters
+            are unknown
+
         Returns:
             The OriginVisitStatus matching the criteria
 
@@ -958,14 +956,14 @@ class StorageInterface:
         ...
 
     @remote_api_endpoint("object/find_by_sha1_git")
-    def object_find_by_sha1_git(self, ids):
+    def object_find_by_sha1_git(self, ids: List[Sha1Git]) -> Dict[Sha1Git, List[Dict]]:
         """Return the objects found with the given ids.
 
         Args:
             ids: a generator of sha1_gits
 
         Returns:
-            dict: a mapping from id to the list of objects found. Each object
+            A dict from id to the list of objects found for that id. Each object
             found is itself a dict with keys:
 
             - sha1_git: the input id
@@ -989,38 +987,18 @@ class StorageInterface:
         ...
 
     @remote_api_endpoint("origin/get_sha1")
-    def origin_get_by_sha1(self, sha1s):
+    def origin_get_by_sha1(self, sha1s: List[bytes]) -> List[Optional[Dict[str, Any]]]:
         """Return origins, identified by the sha1 of their URLs.
 
         Args:
-            sha1s (list[bytes]): a list of sha1s
+            sha1s: a list of sha1s
 
-        Yields:
-            dicts containing origin information as returned
-            by :meth:`swh.storage.storage.Storage.origin_get`, or None if an
-            origin matching the sha1 is not found.
+        Returns:
+            List of origins dict whose sha1 of their url match, None otherwise.
 
         """
         ...
 
-    @deprecated
-    @remote_api_endpoint("origin/get_range")
-    def origin_get_range(self, origin_from=1, origin_count=100):
-        """Retrieve ``origin_count`` origins whose ids are greater
-        or equal than ``origin_from``.
-
-        Origins are sorted by id before retrieving them.
-
-        Args:
-            origin_from (int): the minimum id of origins to retrieve
-            origin_count (int): the maximum number of origins to retrieve
-
-        Yields:
-            dicts containing origin information as returned
-            by :meth:`swh.storage.storage.Storage.origin_get`.
-        """
-        ...
-
     @remote_api_endpoint("origin/list")
     def origin_list(
         self, page_token: Optional[str] = None, limit: int = 100
@@ -1040,30 +1018,36 @@ class StorageInterface:
 
     @remote_api_endpoint("origin/search")
     def origin_search(
-        self, url_pattern, offset=0, limit=50, regexp=False, with_visit=False
-    ):
+        self,
+        url_pattern: str,
+        page_token: Optional[str] = None,
+        limit: int = 50,
+        regexp: bool = False,
+        with_visit: bool = False,
+    ) -> PagedResult[Origin]:
         """Search for origins whose urls contain a provided string pattern
         or match a provided regular expression.
         The search is performed in a case insensitive way.
 
         Args:
-            url_pattern (str): the string pattern to search for in origin urls
-            offset (int): number of found origins to skip before returning
-                results
-            limit (int): the maximum number of found origins to return
-            regexp (bool): if True, consider the provided pattern as a regular
+            url_pattern: the string pattern to search for in origin urls
+            page_token: opaque token used for pagination
+            limit: the maximum number of found origins to return
+            regexp: if True, consider the provided pattern as a regular
                 expression and return origins whose urls match it
-            with_visit (bool): if True, filter out origins with no visit
+            with_visit: if True, filter out origins with no visit
 
         Yields:
-            dicts containing origin information as returned
-            by :meth:`swh.storage.storage.Storage.origin_get`.
+            PagedResult of Origin
+
         """
         ...
 
     @deprecated
     @remote_api_endpoint("origin/count")
-    def origin_count(self, url_pattern, regexp=False, with_visit=False):
+    def origin_count(
+        self, url_pattern: str, regexp: bool = False, with_visit: bool = False
+    ) -> int:
         """Count origins whose urls contain a provided string pattern
         or match a provided regular expression.
         The pattern search in origin urls is performed in a case insensitive
@@ -1133,28 +1117,25 @@ class StorageInterface:
     @remote_api_endpoint("raw_extrinsic_metadata/get")
     def raw_extrinsic_metadata_get(
         self,
-        object_type: MetadataTargetType,
+        type: MetadataTargetType,
         id: Union[str, SWHID],
         authority: MetadataAuthority,
         after: Optional[datetime.datetime] = None,
         page_token: Optional[bytes] = None,
         limit: int = 1000,
-    ) -> Dict[str, Union[Optional[bytes], List[RawExtrinsicMetadata]]]:
+    ) -> PagedResult[RawExtrinsicMetadata]:
         """Retrieve list of all raw_extrinsic_metadata entries for the id
 
         Args:
-            object_type: one of the values of swh.model.model.MetadataTargetType
-            id: an URL if object_type is 'origin', else a core SWHID
+            type: one of the values of swh.model.model.MetadataTargetType
+            id: an URL if type is 'origin', else a core SWHID
             authority: a dict containing keys `type` and `url`.
             after: minimum discovery_date for a result to be returned
             page_token: opaque token, used to get the next page of results
             limit: maximum number of results to be returned
 
         Returns:
-            dict with keys `next_page_token` and `results`.
-            `next_page_token` is an opaque token that is used to get the
-            next page of results, or `None` if there are no more results.
-            `results` is a list of RawExtrinsicMetadata objects:
+            PagedResult of RawExtrinsicMetadata
 
         """
         ...
