@@ -455,7 +455,8 @@ create type revision_entry as
   metadata                       jsonb,
   synthetic                      boolean,
   parents                        bytea[],
-  object_id                      bigint
+  object_id                      bigint,
+  extra_headers                  bytea[][]
 );
 
 
@@ -471,7 +472,7 @@ as $$
            r.type, r.directory, r.message,
            a.id, a.fullname, a.name, a.email,
            c.id, c.fullname, c.name, c.email,
-           r.metadata, r.synthetic, t.parents, r.object_id
+           r.metadata, r.synthetic, t.parents, r.object_id, r.extra_headers
     from swh_revision_list(root_revisions, num_revs) as t
     left join revision r on t.id = r.id
     left join person a on a.id = r.author
@@ -528,8 +529,8 @@ as $$
 begin
     perform swh_person_add_from_revision();
 
-    insert into revision (id, date, date_offset, date_neg_utc_offset, committer_date, committer_date_offset, committer_date_neg_utc_offset, type, directory, message, author, committer, metadata, synthetic)
-    select t.id, t.date, t.date_offset, t.date_neg_utc_offset, t.committer_date, t.committer_date_offset, t.committer_date_neg_utc_offset, t.type, t.directory, t.message, a.id, c.id, t.metadata, t.synthetic
+    insert into revision (id, date, date_offset, date_neg_utc_offset, committer_date, committer_date_offset, committer_date_neg_utc_offset, type, directory, message, author, committer, metadata, synthetic, extra_headers)
+    select t.id, t.date, t.date_offset, t.date_neg_utc_offset, t.committer_date, t.committer_date_offset, t.committer_date_neg_utc_offset, t.type, t.directory, t.message, a.id, c.id, t.metadata, t.synthetic, t.extra_headers
     from tmp_revision t
     left join person a on a.fullname = t.author_fullname
     left join person c on c.fullname = t.committer_fullname;
@@ -568,9 +569,10 @@ begin
     perform swh_person_add_from_release();
 
     insert into release (id, target, target_type, date, date_offset, date_neg_utc_offset, name, comment, author, synthetic)
-    select t.id, t.target, t.target_type, t.date, t.date_offset, t.date_neg_utc_offset, t.name, t.comment, a.id, t.synthetic
-    from tmp_release t
-    left join person a on a.fullname = t.author_fullname;
+      select distinct t.id, t.target, t.target_type, t.date, t.date_offset, t.date_neg_utc_offset, t.name, t.comment, a.id, t.synthetic
+        from tmp_release t
+        left join person a on a.fullname = t.author_fullname
+        where not exists (select 1 from release where t.id = release.id);
     return;
 end
 $$;
@@ -592,9 +594,9 @@ as $$
     from origin_visit
     where origin = (select id from origin_id)
   )
-  insert into origin_visit (origin, date, type, visit, status)
+  insert into origin_visit (origin, date, type, visit)
   values ((select id from origin_id), date, type,
-          (select visit from last_known_visit) + 1, 'ongoing')
+          (select visit from last_known_visit) + 1)
   returning visit;
 $$;
 
@@ -650,14 +652,25 @@ create or replace function swh_snapshot_get_by_id(id sha1_git,
   language sql
   stable
 as $$
-  select
-    swh_snapshot_get_by_id.id as snapshot_id, name, target, target_type
-  from snapshot_branches
-  inner join snapshot_branch on snapshot_branches.branch_id = snapshot_branch.object_id
-  where snapshot_id = (select object_id from snapshot where snapshot.id = swh_snapshot_get_by_id.id)
-    and (target_types is null or target_type = any(target_types))
-    and name >= branches_from
-  order by name limit branches_count
+  -- with small limits, the "naive" version of this query can degenerate into
+  -- using the deduplication index on snapshot_branch (name, target,
+  -- target_type); The planner happily scans several hundred million rows.
+
+  -- Do the query in two steps: first pull the relevant branches for the given
+  -- snapshot (filtering them by type), then do the limiting. This two-step
+  -- process guides the planner into using the proper index.
+  with filtered_snapshot_branches as (
+    select swh_snapshot_get_by_id.id as snapshot_id, name, target, target_type
+      from snapshot_branches
+      inner join snapshot_branch on snapshot_branches.branch_id = snapshot_branch.object_id
+      where snapshot_id = (select object_id from snapshot where snapshot.id = swh_snapshot_get_by_id.id)
+        and (target_types is null or target_type = any(target_types))
+      order by name
+  )
+  select snapshot_id, name, target, target_type
+    from filtered_snapshot_branches
+    where name >= branches_from
+    order by name limit branches_count;
 $$;
 
 create type snapshot_size as (
@@ -781,7 +794,7 @@ as $$
            r.type, r.directory, r.message,
            a.id, a.fullname, a.name, a.email, c.id, c.fullname, c.name, c.email, r.metadata, r.synthetic,
            array(select rh.parent_id::bytea from revision_history rh where rh.id = r.id order by rh.parent_rank)
-               as parents, r.object_id
+               as parents, r.object_id, r.extra_headers
     from revs r
     left join person a on a.id = r.author
     left join person c on c.id = r.committer
