@@ -3,6 +3,7 @@
 # License: GNU General Public License version 3, or any later version
 # See top-level LICENSE file for more information
 
+import base64
 import contextlib
 import datetime
 import itertools
@@ -10,6 +11,7 @@ import itertools
 from collections import defaultdict
 from contextlib import contextmanager
 from typing import (
+    Any,
     Counter,
     Dict,
     Iterable,
@@ -25,7 +27,7 @@ import psycopg2.pool
 import psycopg2.errors
 
 from swh.core.api.serializers import msgpack_loads, msgpack_dumps
-from swh.model.identifiers import parse_swhid, SWHID
+from swh.model.identifiers import SWHID
 from swh.model.model import (
     Content,
     Directory,
@@ -35,6 +37,8 @@ from swh.model.model import (
     Revision,
     Release,
     SkippedContent,
+    Sha1,
+    Sha1Git,
     Snapshot,
     SHA1_SIZE,
     MetadataAuthority,
@@ -44,7 +48,7 @@ from swh.model.model import (
     RawExtrinsicMetadata,
 )
 from swh.model.hashutil import DEFAULT_ALGORITHMS, hash_to_bytes, hash_to_hex
-from swh.storage.interface import ListOrder, PagedResult
+from swh.storage.interface import ListOrder, PagedResult, VISIT_STATUSES
 from swh.storage.objstorage import ObjStorage
 from swh.storage.utils import now
 
@@ -141,7 +145,7 @@ class Storage:
 
     @timed
     @db_transaction()
-    def check_config(self, *, check_write, db=None, cur=None):
+    def check_config(self, *, check_write: bool, db=None, cur=None) -> bool:
 
         if not self.objstorage.check_config(check_write=check_write):
             return False
@@ -233,15 +237,17 @@ class Storage:
 
     @timed
     @db_transaction()
-    def content_update(self, content, keys=[], db=None, cur=None):
+    def content_update(
+        self, contents: List[Dict[str, Any]], keys: List[str] = [], db=None, cur=None
+    ) -> None:
         # TODO: Add a check on input keys. How to properly implement
         # this? We don't know yet the new columns.
-        self.journal_writer.content_update(content)
+        self.journal_writer.content_update(contents)
 
         db.mktemp("content", cur)
         select_keys = list(set(db.content_get_metadata_keys).union(set(keys)))
         with convert_validation_exceptions():
-            db.copy_to(content, "tmp_content", select_keys, cur)
+            db.copy_to(contents, "tmp_content", select_keys, cur)
             db.content_update_from_temp(keys_to_update=keys, cur=cur)
 
     @timed
@@ -261,43 +267,21 @@ class Storage:
         }
 
     @timed
-    def content_get(self, content):
-        # FIXME: Make this method support slicing the `data`.
-        if len(content) > BULK_BLOCK_CONTENT_LEN_MAX:
-            raise StorageArgumentException(
-                "Send at maximum %s contents." % BULK_BLOCK_CONTENT_LEN_MAX
-            )
-        yield from self.objstorage.content_get(content)
+    def content_get_data(self, content: Sha1) -> Optional[bytes]:
+        # FIXME: Make this method support slicing the `data`
+        return self.objstorage.content_get(content)
 
     @timed
     @db_transaction()
-    def content_get_range(self, start, end, limit=1000, db=None, cur=None):
-        if limit is None:
-            raise StorageArgumentException("limit should not be None")
-        contents = []
-        next_content = None
-        for counter, content_row in enumerate(
-            db.content_get_range(start, end, limit + 1, cur)
-        ):
-            content = dict(zip(db.content_get_metadata_keys, content_row))
-            if counter >= limit:
-                # take the last commit for the next page starting from this
-                next_content = content["sha1"]
-                break
-            contents.append(content)
-        return {
-            "contents": contents,
-            "next": next_content,
-        }
-
-    @timed
     def content_get_partition(
         self,
         partition_id: int,
         nb_partitions: int,
+        page_token: Optional[str] = None,
         limit: int = 1000,
-        page_token: str = None,
-    ):
+        db=None,
+        cur=None,
+    ) -> PagedResult[Content]:
         if limit is None:
             raise StorageArgumentException("limit should not be None")
         (start, end) = get_partition_bounds_bytes(
@@ -307,75 +291,91 @@ class Storage:
             start = hash_to_bytes(page_token)
         if end is None:
             end = b"\xff" * SHA1_SIZE
-        result = self.content_get_range(start, end, limit)
-        result2 = {
-            "contents": result["contents"],
-            "next_page_token": None,
-        }
-        if result["next"]:
-            result2["next_page_token"] = hash_to_hex(result["next"])
-        return result2
+
+        next_page_token: Optional[str] = None
+        contents = []
+        for counter, row in enumerate(db.content_get_range(start, end, limit + 1, cur)):
+            row_d = dict(zip(db.content_get_metadata_keys, row))
+            content = Content(**row_d)
+            if counter >= limit:
+                # take the last content for the next page starting from this
+                next_page_token = hash_to_hex(content.sha1)
+                break
+            contents.append(content)
+
+        assert len(contents) <= limit
+        return PagedResult(results=contents, next_page_token=next_page_token)
 
     @timed
     @db_transaction(statement_timeout=500)
-    def content_get_metadata(
-        self, contents: List[bytes], db=None, cur=None
-    ) -> Dict[bytes, List[Dict]]:
-        result: Dict[bytes, List[Dict]] = {sha1: [] for sha1 in contents}
+    def content_get(
+        self, contents: List[Sha1], db=None, cur=None
+    ) -> List[Optional[Content]]:
+        contents_by_sha1: Dict[Sha1, Optional[Content]] = {}
         for row in db.content_get_metadata_from_sha1s(contents, cur):
-            content_meta = dict(zip(db.content_get_metadata_keys, row))
-            result[content_meta["sha1"]].append(content_meta)
-        return result
+            row_d = dict(zip(db.content_get_metadata_keys, row))
+            content = Content(**row_d)
+            contents_by_sha1[content.sha1] = content
+
+        return [contents_by_sha1.get(sha1) for sha1 in contents]
 
     @timed
     @db_transaction_generator()
-    def content_missing(self, content, key_hash="sha1", db=None, cur=None):
+    def content_missing(
+        self, contents: List[Dict[str, Any]], key_hash: str = "sha1", db=None, cur=None
+    ) -> Iterable[bytes]:
+        if key_hash not in DEFAULT_ALGORITHMS:
+            raise StorageArgumentException(
+                "key_hash should be one of {','.join(DEFAULT_ALGORITHMS)}"
+            )
+
         keys = db.content_hash_keys
-
-        if key_hash not in keys:
-            raise StorageArgumentException("key_hash should be one of %s" % keys)
-
         key_hash_idx = keys.index(key_hash)
 
-        if not content:
-            return
-
-        for obj in db.content_missing_from_list(content, cur):
+        for obj in db.content_missing_from_list(contents, cur):
             yield obj[key_hash_idx]
 
     @timed
     @db_transaction_generator()
-    def content_missing_per_sha1(self, contents, db=None, cur=None):
+    def content_missing_per_sha1(
+        self, contents: List[bytes], db=None, cur=None
+    ) -> Iterable[bytes]:
         for obj in db.content_missing_per_sha1(contents, cur):
             yield obj[0]
 
     @timed
     @db_transaction_generator()
-    def content_missing_per_sha1_git(self, contents, db=None, cur=None):
+    def content_missing_per_sha1_git(
+        self, contents: List[bytes], db=None, cur=None
+    ) -> Iterable[Sha1Git]:
         for obj in db.content_missing_per_sha1_git(contents, cur):
             yield obj[0]
 
     @timed
     @db_transaction()
-    def content_find(self, content, db=None, cur=None):
+    def content_find(self, content: Dict[str, Any], db=None, cur=None) -> List[Content]:
         if not set(content).intersection(DEFAULT_ALGORITHMS):
             raise StorageArgumentException(
-                "content keys must contain at least one of: "
-                "sha1, sha1_git, sha256, blake2s256"
+                "content keys must contain at least one "
+                f"of: {', '.join(sorted(DEFAULT_ALGORITHMS))}"
             )
 
-        contents = db.content_find(
+        rows = db.content_find(
             sha1=content.get("sha1"),
             sha1_git=content.get("sha1_git"),
             sha256=content.get("sha256"),
             blake2s256=content.get("blake2s256"),
             cur=cur,
         )
-        return [dict(zip(db.content_find_cols, content)) for content in contents]
+        contents = []
+        for row in rows:
+            row_d = dict(zip(db.content_find_cols, row))
+            contents.append(Content(**row_d))
+        return contents
 
     @timed
     @db_transaction()
-    def content_get_random(self, db=None, cur=None):
+    def content_get_random(self, db=None, cur=None) -> Sha1Git:
         return db.content_get_random(cur)
 
     @staticmethod
@@ -440,7 +440,9 @@ class Storage:
 
     @timed
     @db_transaction_generator()
-    def skipped_content_missing(self, contents, db=None, cur=None):
+    def skipped_content_missing(
+        self, contents: List[Dict[str, Any]], db=None, cur=None
+    ) -> Iterable[Dict[str, Any]]:
         contents = list(contents)
         for content in db.skipped_content_missing(contents, cur):
             yield dict(zip(db.content_hash_keys, content))
@@ -504,13 +506,17 @@ class Storage:
 
     @timed
     @db_transaction_generator()
-    def directory_missing(self, directories, db=None, cur=None):
+    def directory_missing(
+        self, directories: List[Sha1Git], db=None, cur=None
+    ) -> Iterable[Sha1Git]:
         for obj in db.directory_missing_from_list(directories, cur):
             yield obj[0]
 
     @timed
     @db_transaction_generator(statement_timeout=20000)
-    def directory_ls(self, directory, recursive=False, db=None, cur=None):
+    def directory_ls(
+        self, directory: Sha1Git, recursive: bool = False, db=None, cur=None
+    ) -> Iterable[Dict[str, Any]]:
         if recursive:
             res_gen = db.directory_walk(directory, cur=cur)
         else:
@@ -521,14 +527,15 @@ class Storage:
 
     @timed
     @db_transaction(statement_timeout=2000)
-    def directory_entry_get_by_path(self, directory, paths, db=None, cur=None):
+    def directory_entry_get_by_path(
+        self, directory: Sha1Git, paths: List[bytes], db=None, cur=None
+    ) -> Optional[Dict[str, Any]]:
         res = db.directory_entry_get_by_path(directory, paths, cur)
-        if res:
-            return dict(zip(db.directory_ls_cols, res))
+        return dict(zip(db.directory_ls_cols, res)) if res else None
 
     @timed
     @db_transaction()
-    def directory_get_random(self, db=None, cur=None):
+    def directory_get_random(self, db=None, cur=None) -> Sha1Git:
         return db.directory_get_random(cur)
 
     @timed
@@ -580,16 +587,20 @@ class Storage:
 
     @timed
     @db_transaction_generator()
-    def revision_missing(self, revisions, db=None, cur=None):
+    def revision_missing(
+        self, revisions: List[Sha1Git], db=None, cur=None
+    ) -> Iterable[Sha1Git]:
         if not revisions:
-            return
+            return None
 
         for obj in db.revision_missing_from_list(revisions, cur):
             yield obj[0]
 
     @timed
     @db_transaction_generator(statement_timeout=1000)
-    def revision_get(self, revisions, db=None, cur=None):
+    def revision_get(
+        self, revisions: List[Sha1Git], db=None, cur=None
+    ) -> Iterable[Optional[Dict[str, Any]]]:
         for line in db.revision_get_from_list(revisions, cur):
             data = converters.db_to_revision(dict(zip(db.revision_get_cols, line)))
             if not data["type"]:
@@ -599,7 +610,9 @@ class Storage:
 
     @timed
     @db_transaction_generator(statement_timeout=2000)
-    def revision_log(self, revisions, limit=None, db=None, cur=None):
+    def revision_log(
+        self, revisions: List[Sha1Git], limit: Optional[int] = None, db=None, cur=None
+    ) -> Iterable[Optional[Dict[str, Any]]]:
         for line in db.revision_log(revisions, limit, cur):
             data = converters.db_to_revision(dict(zip(db.revision_get_cols, line)))
             if not data["type"]:
@@ -609,13 +622,14 @@ class Storage:
 
     @timed
     @db_transaction_generator(statement_timeout=2000)
-    def revision_shortlog(self, revisions, limit=None, db=None, cur=None):
-
+    def revision_shortlog(
+        self, revisions: List[Sha1Git], limit: Optional[int] = None, db=None, cur=None
+    ) -> Iterable[Optional[Tuple[Sha1Git, Tuple[Sha1Git, ...]]]]:
         yield from db.revision_shortlog(revisions, limit, cur)
 
     @timed
     @db_transaction()
-    def revision_get_random(self, db=None, cur=None):
+    def revision_get_random(self, db=None, cur=None) -> Sha1Git:
         return db.revision_get_random(cur)
 
     @timed
@@ -649,7 +663,9 @@ class Storage:
 
     @timed
     @db_transaction_generator()
-    def release_missing(self, releases, db=None, cur=None):
+    def release_missing(
+        self, releases: List[Sha1Git], db=None, cur=None
+    ) -> Iterable[Sha1Git]:
         if not releases:
             return
 
@@ -658,14 +674,16 @@ class Storage:
 
     @timed
     @db_transaction_generator(statement_timeout=500)
-    def release_get(self, releases, db=None, cur=None):
+    def release_get(
+        self, releases: List[Sha1Git], db=None, cur=None
+    ) -> Iterable[Optional[Dict[str, Any]]]:
         for release in db.release_get_from_list(releases, cur):
             data = converters.db_to_release(dict(zip(db.release_get_cols, release)))
             yield data if data["target_type"] else None
 
     @timed
     @db_transaction()
-    def release_get_random(self, db=None, cur=None):
+    def release_get_random(self, db=None, cur=None) -> Sha1Git:
         return db.release_get_random(cur)
 
     @timed
@@ -707,19 +725,24 @@ class Storage:
 
     @timed
     @db_transaction_generator()
-    def snapshot_missing(self, snapshots, db=None, cur=None):
+    def snapshot_missing(
+        self, snapshots: List[Sha1Git], db=None, cur=None
+    ) -> Iterable[Sha1Git]:
         for obj in db.snapshot_missing_from_list(snapshots, cur):
             yield obj[0]
 
     @timed
     @db_transaction(statement_timeout=2000)
-    def snapshot_get(self, snapshot_id, db=None, cur=None):
-
+    def snapshot_get(
+        self, snapshot_id: Sha1Git, db=None, cur=None
+    ) -> Optional[Dict[str, Any]]:
         return self.snapshot_get_branches(snapshot_id, db=db, cur=cur)
 
     @timed
     @db_transaction(statement_timeout=2000)
-    def snapshot_get_by_origin_visit(self, origin, visit, db=None, cur=None):
+    def snapshot_get_by_origin_visit(
+        self, origin: str, visit: int, db=None, cur=None
+    ) -> Optional[Dict[str, Any]]:
         snapshot_id = db.snapshot_get_by_origin_visit(origin, visit, cur)
 
         if snapshot_id:
@@ -729,20 +752,22 @@ class Storage:
 
     @timed
     @db_transaction(statement_timeout=2000)
-    def snapshot_count_branches(self, snapshot_id, db=None, cur=None):
+    def snapshot_count_branches(
+        self, snapshot_id: Sha1Git, db=None, cur=None
+    ) -> Optional[Dict[str, int]]:
         return dict([bc for bc in db.snapshot_count_branches(snapshot_id, cur)])
 
     @timed
     @db_transaction(statement_timeout=2000)
     def snapshot_get_branches(
         self,
-        snapshot_id,
-        branches_from=b"",
-        branches_count=1000,
-        target_types=None,
+        snapshot_id: Sha1Git,
+        branches_from: bytes = b"",
+        branches_count: int = 1000,
+        target_types: Optional[List[str]] = None,
         db=None,
         cur=None,
-    ):
+    ) -> Optional[Dict[str, Any]]:
         if snapshot_id == EMPTY_SNAPSHOT_ID:
             return {
                 "id": snapshot_id,
@@ -785,7 +810,7 @@ class Storage:
 
     @timed
     @db_transaction()
-    def snapshot_get_random(self, db=None, cur=None):
+    def snapshot_get_random(self, db=None, cur=None) -> Sha1Git:
         return db.snapshot_get_random(cur)
 
     @timed
@@ -861,6 +886,12 @@ class Storage:
         db=None,
         cur=None,
     ) -> Optional[OriginVisitStatus]:
+        if allowed_statuses and not set(allowed_statuses).intersection(VISIT_STATUSES):
+            raise StorageArgumentException(
+                f"Unknown allowed statuses {','.join(allowed_statuses)}, only "
+                f"{','.join(VISIT_STATUSES)} authorized"
+            )
+
         row = db.origin_visit_status_get_latest(
             origin_url, visit, allowed_statuses, require_snapshot, cur=cur
         )
@@ -952,6 +983,12 @@ class Storage:
         db=None,
         cur=None,
     ) -> Optional[OriginVisit]:
+        if allowed_statuses and not set(allowed_statuses).intersection(VISIT_STATUSES):
+            raise StorageArgumentException(
+                f"Unknown allowed statuses {','.join(allowed_statuses)}, only "
+                f"{','.join(VISIT_STATUSES)} authorized"
+            )
+
         row = db.origin_visit_get_latest(
             origin,
             type=type,
@@ -1039,8 +1076,10 @@ class Storage:
 
     @timed
     @db_transaction(statement_timeout=2000)
-    def object_find_by_sha1_git(self, ids, db=None, cur=None):
-        ret = {id: [] for id in ids}
+    def object_find_by_sha1_git(
+        self, ids: List[Sha1Git], db=None, cur=None
+    ) -> Dict[Sha1Git, List[Dict]]:
+        ret: Dict[Sha1Git, List[Dict]] = {id: [] for id in ids}
 
         for retval in db.object_find_by_sha1_git(ids, cur=cur):
             if retval[1]:
@@ -1064,13 +1103,14 @@ class Storage:
         return result
 
     @timed
-    @db_transaction_generator(statement_timeout=500)
-    def origin_get_by_sha1(self, sha1s, db=None, cur=None):
-        for line in db.origin_get_by_sha1(sha1s, cur):
-            if line[0] is not None:
-                yield dict(zip(db.origin_cols, line))
-            else:
-                yield None
+    @db_transaction(statement_timeout=500)
+    def origin_get_by_sha1(
+        self, sha1s: List[bytes], db=None, cur=None
+    ) -> List[Optional[Dict[str, Any]]]:
+        return [
+            dict(zip(db.origin_cols, row)) if row[0] else None
+            for row in db.origin_get_by_sha1(sha1s, cur)
+        ]
 
     @timed
     @db_transaction_generator()
@@ -1106,27 +1146,48 @@ class Storage:
         return PagedResult(results=origins, next_page_token=next_page_token)
 
     @timed
-    @db_transaction_generator()
+    @db_transaction()
     def origin_search(
         self,
-        url_pattern,
-        offset=0,
-        limit=50,
-        regexp=False,
-        with_visit=False,
+        url_pattern: str,
+        page_token: Optional[str] = None,
+        limit: int = 50,
+        regexp: bool = False,
+        with_visit: bool = False,
         db=None,
         cur=None,
-    ):
+    ) -> PagedResult[Origin]:
+        next_page_token = None
+        offset = int(page_token) if page_token else 0
+
+        origins = []
+        # Take one more origin so we can reuse it as the next page token if any
         for origin in db.origin_search(
-            url_pattern, offset, limit, regexp, with_visit, cur
+            url_pattern, offset, limit + 1, regexp, with_visit, cur
         ):
-            yield dict(zip(db.origin_cols, origin))
+            row_d = dict(zip(db.origin_cols, origin))
+            origins.append(Origin(url=row_d["url"]))
+
+        if len(origins) > limit:
+            # next offset
+            next_page_token = str(offset + limit)
+            # excluding that origin from the result to respect the limit size
+            origins = origins[:limit]
+
+        assert len(origins) <= limit
+
+        return PagedResult(results=origins, next_page_token=next_page_token)
 
     @timed
     @db_transaction()
     def origin_count(
-        self, url_pattern, regexp=False, with_visit=False, db=None, cur=None
-    ):
+        self,
+        url_pattern: str,
+        regexp: bool = False,
+        with_visit: bool = False,
+        db=None,
+        cur=None,
+    ) -> int:
         return db.origin_count(url_pattern, regexp, with_visit, cur)
 
     @timed
@@ -1182,7 +1243,7 @@ class Storage:
             fetcher_id = self._get_fetcher_id(metadata_entry.fetcher, db, cur)
 
             db.raw_extrinsic_metadata_add(
-                object_type=metadata_entry.type.value,
+                type=metadata_entry.type.value,
                 id=str(metadata_entry.id),
                 discovery_date=metadata_entry.discovery_date,
                 authority_id=authority_id,
@@ -1200,17 +1261,17 @@ class Storage:
             )
             counter[metadata_entry.type] += 1
 
-        for (object_type, count) in counter.items():
+        for (type, count) in counter.items():
             send_metric(
-                f"{object_type.value}_metadata:add",
+                f"{type.value}_metadata:add",
                 count=count,
-                method_name=f"{object_type.value}_metadata_add",
+                method_name=f"{type.value}_metadata_add",
             )
 
     @db_transaction()
     def raw_extrinsic_metadata_get(
         self,
-        object_type: MetadataTargetType,
+        type: MetadataTargetType,
         id: Union[str, SWHID],
         authority: MetadataAuthority,
         after: Optional[datetime.datetime] = None,
@@ -1218,22 +1279,22 @@ class Storage:
         limit: int = 1000,
         db=None,
         cur=None,
-    ) -> Dict[str, Union[Optional[bytes], List[RawExtrinsicMetadata]]]:
-        if object_type == MetadataTargetType.ORIGIN:
+    ) -> PagedResult[RawExtrinsicMetadata]:
+        if type == MetadataTargetType.ORIGIN:
             if isinstance(id, SWHID):
                 raise StorageArgumentException(
-                    f"raw_extrinsic_metadata_get called with object_type='origin', "
+                    f"raw_extrinsic_metadata_get called with type='origin', "
                     f"but provided id is an SWHID: {id!r}"
                 )
         else:
             if not isinstance(id, SWHID):
                 raise StorageArgumentException(
-                    f"raw_extrinsic_metadata_get called with object_type!='origin', "
+                    f"raw_extrinsic_metadata_get called with type!='origin', "
                     f"but provided id is not an SWHID: {id!r}"
                 )
 
         if page_token:
-            (after_time, after_fetcher) = msgpack_loads(page_token)
+            (after_time, after_fetcher) = msgpack_loads(base64.b64decode(page_token))
             if after and after_time < after:
                 raise StorageArgumentException(
                     "page_token is inconsistent with the value of 'after'."
@@ -1244,70 +1305,33 @@ class Storage:
 
         authority_id = self._get_authority_id(authority, db, cur)
         if not authority_id:
-            return {
-                "next_page_token": None,
-                "results": [],
-            }
+            return PagedResult(next_page_token=None, results=[],)
 
         rows = db.raw_extrinsic_metadata_get(
-            object_type,
-            str(id),
-            authority_id,
-            after_time,
-            after_fetcher,
-            limit + 1,
-            cur,
+            type, str(id), authority_id, after_time, after_fetcher, limit + 1, cur,
         )
         rows = [dict(zip(db.raw_extrinsic_metadata_get_cols, row)) for row in rows]
         results = []
         for row in rows:
-            row = row.copy()
-            row.pop("metadata_fetcher.id")
-
             assert str(id) == row["raw_extrinsic_metadata.id"]
-
-            result = RawExtrinsicMetadata(
-                type=MetadataTargetType(row["raw_extrinsic_metadata.type"]),
-                id=id,
-                authority=MetadataAuthority(
-                    type=MetadataAuthorityType(row["metadata_authority.type"]),
-                    url=row["metadata_authority.url"],
-                ),
-                fetcher=MetadataFetcher(
-                    name=row["metadata_fetcher.name"],
-                    version=row["metadata_fetcher.version"],
-                ),
-                discovery_date=row["discovery_date"],
-                format=row["format"],
-                metadata=row["raw_extrinsic_metadata.metadata"],
-                origin=row["origin"],
-                visit=row["visit"],
-                snapshot=map_optional(parse_swhid, row["snapshot"]),
-                release=map_optional(parse_swhid, row["release"]),
-                revision=map_optional(parse_swhid, row["revision"]),
-                path=row["path"],
-                directory=map_optional(parse_swhid, row["directory"]),
-            )
-
-            results.append(result)
+            results.append(converters.db_to_raw_extrinsic_metadata(row))
 
         if len(results) > limit:
             results.pop()
             assert len(results) == limit
             last_returned_row = rows[-2]  # rows[-1] corresponds to the popped result
-            next_page_token: Optional[bytes] = msgpack_dumps(
-                (
-                    last_returned_row["discovery_date"],
-                    last_returned_row["metadata_fetcher.id"],
+            next_page_token: Optional[str] = base64.b64encode(
+                msgpack_dumps(
+                    (
+                        last_returned_row["discovery_date"],
+                        last_returned_row["metadata_fetcher.id"],
+                    )
                 )
-            )
+            ).decode()
         else:
             next_page_token = None
 
-        return {
-            "next_page_token": next_page_token,
-            "results": results,
-        }
+        return PagedResult(next_page_token=next_page_token, results=results,)
 
     @timed
     @db_transaction()
