@@ -10,7 +10,11 @@ from swh.model.collections import ImmutableDict
 from swh.model.hypothesis_strategies import snapshots, branch_names, branch_targets
 from swh.model.model import OriginVisit, OriginVisitStatus, Snapshot
 
-from swh.storage.algos.snapshot import snapshot_get_all_branches, snapshot_get_latest
+from swh.storage.algos.snapshot import (
+    snapshot_get_all_branches,
+    snapshot_get_latest,
+    snapshot_id_get_from_revision,
+)
 from swh.storage.utils import now
 
 
@@ -145,3 +149,61 @@ def test_snapshot_get_latest(swh_storage, sample_data):
 
     with pytest.raises(ValueError, match="branches_count must be a positive integer"):
         snapshot_get_latest(swh_storage, origin.url, branches_count="something-wrong")
+
+
+def test_snapshot_get_id_from_revision(swh_storage, sample_data):
+    origin = sample_data.origin
+    swh_storage.origin_add([origin])
+
+    date_visit2 = now()
+    visit1, visit2 = sample_data.origin_visits[:2]
+    assert visit1.origin == origin.url
+
+    ov1, ov2 = swh_storage.origin_visit_add([visit1, visit2])
+
+    revision1, revision2, revision3 = sample_data.revisions[:3]
+    swh_storage.revision_add([revision1, revision2])
+
+    empty_snapshot, complete_snapshot = sample_data.snapshots[1:3]
+    swh_storage.snapshot_add([complete_snapshot])
+
+    # Add complete_snapshot to visit1 which targets revision1
+    ovs1, ovs2 = [
+        OriginVisitStatus(
+            origin=origin.url,
+            visit=ov1.visit,
+            date=date_visit2,
+            status="partial",
+            snapshot=complete_snapshot.id,
+        ),
+        OriginVisitStatus(
+            origin=origin.url,
+            visit=ov2.visit,
+            date=now(),
+            status="full",
+            snapshot=empty_snapshot.id,
+        ),
+    ]
+
+    swh_storage.origin_visit_status_add([ovs1, ovs2])
+    assert ov1.date < ov2.date
+    assert ov2.date < ovs1.date
+    assert ovs1.date < ovs2.date
+
+    # revision3 does not exist so result is None
+    actual_snapshot_id = snapshot_id_get_from_revision(
+        swh_storage, origin.url, revision3.id
+    )
+    assert actual_snapshot_id is None
+
+    # no snapshot targets revision2 for origin.url so result is None
+    actual_snapshot_id = snapshot_id_get_from_revision(
+        swh_storage, origin.url, revision2.id
+    )
+    assert actual_snapshot_id is None
+
+    # complete_snapshot targets at least revision1
+    actual_snapshot_id = snapshot_id_get_from_revision(
+        swh_storage, origin.url, revision1.id
+    )
+    assert actual_snapshot_id == complete_snapshot.id
