@@ -34,7 +34,10 @@ def app(app_server):
 @pytest.fixture
 def swh_rpc_client_class():
     def storage_factory(**kwargs):
-        storage_config = {"cls": "validate", "storage": {"cls": "remote", **kwargs,}}
+        storage_config = {
+            "cls": "remote",
+            **kwargs,
+        }
         return get_storage(**storage_config)
 
     return storage_factory
@@ -60,8 +63,8 @@ def swh_storage(swh_rpc_client, app_server):
 
 
 class TestStorage(_TestStorage):
-    def test_content_update(self, swh_storage, app_server):
+    def test_content_update(self, swh_storage, app_server, sample_data):
         # TODO, journal_writer not supported
         swh_storage.journal_writer.journal = None
         with patch.object(server.storage.journal_writer, "journal", None):
-            super().test_content_update(swh_storage)
+            super().test_content_update(swh_storage, sample_data)
