@@ -1,29 +1,31 @@
-# Copyright (C) 2018-2019  The Software Heritage developers
+# Copyright (C) 2018-2020  The Software Heritage developers
 # See the AUTHORS file at the top-level directory of this distribution
 # License: GNU General Public License version 3, or any later version
 # See top-level LICENSE file for more information
 
-from collections import namedtuple
-import datetime
+import attr
 import os
 import signal
 import socket
 import subprocess
 import time
 
+from collections import namedtuple
+from typing import Dict
+
 import pytest
 
+from swh.core.api.classes import stream_results
 from swh.storage import get_storage
 from swh.storage.cassandra import create_keyspace
 from swh.storage.cassandra.schema import TABLES, HASH_ALGORITHMS
 
+from swh.storage.utils import now
 from swh.storage.tests.test_storage import TestStorage as _TestStorage
 from swh.storage.tests.test_storage import (
     TestStorageGeneratedData as _TestStorageGeneratedData,
 )
 
-from .storage_data import data
-
 
 CONFIG_TEMPLATE = """
 data_file_directories:
@@ -185,7 +187,7 @@ def swh_storage_backend_config(cassandra_cluster, keyspace):
 
 @pytest.mark.cassandra
 class TestCassandraStorage(_TestStorage):
-    def test_content_add_murmur3_collision(self, swh_storage, mocker):
+    def test_content_add_murmur3_collision(self, swh_storage, mocker, sample_data):
         """The Murmur3 token is used as link from index tables to the main
         table; and non-matching contents with colliding murmur3-hash
         are filtered-out when reading the main table.
@@ -193,6 +195,8 @@ class TestCassandraStorage(_TestStorage):
         """
         called = 0
 
+        cont, cont2 = sample_data.contents[:2]
+
         # always return a token
         def mock_cgtfsh(algo, hash_):
             nonlocal called
@@ -201,39 +205,41 @@ class TestCassandraStorage(_TestStorage):
             return [123456]
 
         mocker.patch.object(
-            swh_storage.storage._cql_runner,
-            "content_get_tokens_from_single_hash",
-            mock_cgtfsh,
+            swh_storage._cql_runner, "content_get_tokens_from_single_hash", mock_cgtfsh,
         )
 
-        # For all tokens, always return data.cont
+        # For all tokens, always return cont
         Row = namedtuple("Row", HASH_ALGORITHMS)
 
         def mock_cgft(token):
             nonlocal called
             called += 1
-            return [Row(**{algo: data.cont[algo] for algo in HASH_ALGORITHMS})]
+            return [Row(**{algo: getattr(cont, algo) for algo in HASH_ALGORITHMS})]
 
         mocker.patch.object(
-            swh_storage.storage._cql_runner, "content_get_from_token", mock_cgft
+            swh_storage._cql_runner, "content_get_from_token", mock_cgft
         )
 
-        actual_result = swh_storage.content_add([data.cont2])
+        actual_result = swh_storage.content_add([cont2])
 
         assert called == 4
         assert actual_result == {
             "content:add": 1,
-            "content:add:bytes": data.cont2["length"],
+            "content:add:bytes": cont2.length,
         }
 
-    def test_content_get_metadata_murmur3_collision(self, swh_storage, mocker):
+    def test_content_get_metadata_murmur3_collision(
+        self, swh_storage, mocker, sample_data
+    ):
         """The Murmur3 token is used as link from index tables to the main
         table; and non-matching contents with colliding murmur3-hash
         are filtered-out when reading the main table.
-        This test checks the content methods do filter out these collision.
+        This test checks the content methods do filter out these collisions.
         """
         called = 0
 
+        cont, cont2 = [attr.evolve(c, ctime=now()) for c in sample_data.contents[:2]]
+
         # always return a token
         def mock_cgtfsh(algo, hash_):
             nonlocal called
@@ -242,85 +248,131 @@ class TestCassandraStorage(_TestStorage):
             return [123456]
 
         mocker.patch.object(
-            swh_storage.storage._cql_runner,
-            "content_get_tokens_from_single_hash",
-            mock_cgtfsh,
+            swh_storage._cql_runner, "content_get_tokens_from_single_hash", mock_cgtfsh,
         )
 
-        # For all tokens, always return data.cont and data.cont2
-        cols = list(set(data.cont) - {"data"})
-        Row = namedtuple("Row", cols + ["ctime"])
+        # For all tokens, always return cont and cont2
+        cols = list(set(cont.to_dict()) - {"data"})
+        Row = namedtuple("Row", cols)
 
         def mock_cgft(token):
             nonlocal called
             called += 1
             return [
-                Row(ctime=42, **{col: cont[col] for col in cols})
-                for cont in [data.cont, data.cont2]
+                Row(**{col: getattr(cont, col) for col in cols})
+                for cont in [cont, cont2]
             ]
 
         mocker.patch.object(
-            swh_storage.storage._cql_runner, "content_get_from_token", mock_cgft
+            swh_storage._cql_runner, "content_get_from_token", mock_cgft
         )
 
-        expected_cont = data.cont.copy()
-        del expected_cont["data"]
-
-        actual_result = swh_storage.content_get_metadata([data.cont["sha1"]])
-
+        actual_result = swh_storage.content_get([cont.sha1])
         assert called == 2
 
-        # but data.cont2 should be filtered out
-        assert actual_result == {data.cont["sha1"]: [expected_cont]}
+        # dropping extra column not returned
+        expected_cont = attr.evolve(cont, data=None)
 
-    def test_content_find_murmur3_collision(self, swh_storage, mocker):
-        """The Murmur3 token is used as link from index tables to the main
-        table; and non-matching contents with colliding murmur3-hash
-        are filtered-out when reading the main table.
-        This test checks the content methods do filter out these collision.
-        """
-        called = 0
-
-        # always return a token
-        def mock_cgtfsh(algo, hash_):
-            nonlocal called
-            called += 1
-            assert algo in ("sha1", "sha1_git")
-            return [123456]
-
-        mocker.patch.object(
-            swh_storage.storage._cql_runner,
-            "content_get_tokens_from_single_hash",
-            mock_cgtfsh,
-        )
-
-        # For all tokens, always return data.cont and data.cont2
-        cols = list(set(data.cont) - {"data"})
-        Row = namedtuple("Row", cols + ["ctime"])
-
-        def mock_cgft(token):
-            nonlocal called
-            called += 1
-            return [
-                Row(ctime=datetime.datetime.now(), **{col: cont[col] for col in cols})
-                for cont in [data.cont, data.cont2]
-            ]
-
-        mocker.patch.object(
-            swh_storage.storage._cql_runner, "content_get_from_token", mock_cgft
-        )
-
-        expected_cont = data.cont.copy()
-        del expected_cont["data"]
-
-        actual_result = swh_storage.content_find({"sha1": data.cont["sha1"]})
-
-        assert called == 2
-
-        # but data.cont2 should be filtered out
-        del actual_result[0]["ctime"]
+        # but cont2 should be filtered out
         assert actual_result == [expected_cont]
 
+    def test_content_find_murmur3_collision(self, swh_storage, mocker, sample_data):
+        """The Murmur3 token is used as link from index tables to the main
+        table; and non-matching contents with colliding murmur3-hash
+        are filtered-out when reading the main table.
+        This test checks the content methods do filter out these collisions.
+        """
+        called = 0
+
+        cont, cont2 = [attr.evolve(c, ctime=now()) for c in sample_data.contents[:2]]
+
+        # always return a token
+        def mock_cgtfsh(algo, hash_):
+            nonlocal called
+            called += 1
+            assert algo in ("sha1", "sha1_git")
+            return [123456]
+
+        mocker.patch.object(
+            swh_storage._cql_runner, "content_get_tokens_from_single_hash", mock_cgtfsh,
+        )
+
+        # For all tokens, always return cont and cont2
+        cols = list(set(cont.to_dict()) - {"data"})
+        Row = namedtuple("Row", cols)
+
+        def mock_cgft(token):
+            nonlocal called
+            called += 1
+            return [
+                Row(**{col: getattr(cont, col) for col in cols})
+                for cont in [cont, cont2]
+            ]
+
+        mocker.patch.object(
+            swh_storage._cql_runner, "content_get_from_token", mock_cgft
+        )
+
+        expected_content = attr.evolve(cont, data=None)
+
+        actual_result = swh_storage.content_find({"sha1": cont.sha1})
+
+        assert called == 2
+
+        # but cont2 should be filtered out
+        assert actual_result == [expected_content]
+
+    def test_content_get_partition_murmur3_collision(
+        self, swh_storage, mocker, sample_data
+    ):
+        """The Murmur3 token is used as link from index tables to the main table; and
+        non-matching contents with colliding murmur3-hash are filtered-out when reading
+        the main table.
+
+        This test checks the content_get_partition endpoints return all contents, even
+        the collisions.
+
+        """
+        called = 0
+
+        rows: Dict[int, Dict] = {}
+        for tok, content in enumerate(sample_data.contents):
+            cont = attr.evolve(content, data=None, ctime=now())
+            row_d = {**cont.to_dict(), "tok": tok}
+            rows[tok] = row_d
+
+        # For all tokens, always return cont
+        keys = set(["tok"] + list(content.to_dict().keys())).difference(set(["data"]))
+        Row = namedtuple("Row", keys)
+
+        def mock_content_get_token_range(range_start, range_end, limit):
+            nonlocal called
+            called += 1
+
+            for tok in list(rows.keys()) * 3:  # yield multiple times the same tok
+                row_d = rows[tok]
+                yield Row(**row_d)
+
+        mocker.patch.object(
+            swh_storage._cql_runner,
+            "content_get_token_range",
+            mock_content_get_token_range,
+        )
+
+        actual_results = list(
+            stream_results(
+                swh_storage.content_get_partition, partition_id=0, nb_partitions=1
+            )
+        )
+
+        assert called > 0
+
+        # everything is listed, even collisions
+        assert len(actual_results) == 3 * len(sample_data.contents)
+        # as we duplicated the returned results, dropping duplicate should yield
+        # the original length
+        assert len(set(actual_results)) == len(sample_data.contents)
+
     @pytest.mark.skip("content_update is not yet implemented for Cassandra")
     def test_content_update(self):
         pass
@@ -350,38 +402,6 @@ class TestCassandraStorageGeneratedData(_TestStorageGeneratedData):
     def test_origin_count(self):
         pass
 
-    @pytest.mark.skip("Not supported by Cassandra")
-    def test_origin_get_range(self):
-        pass
-
-    @pytest.mark.skip("Not supported by Cassandra")
-    def test_origin_get_range_from_zero(self):
-        pass
-
-    @pytest.mark.skip("Not supported by Cassandra")
-    def test_generate_content_get_range_limit(self):
-        pass
-
-    @pytest.mark.skip("Not supported by Cassandra")
-    def test_generate_content_get_range_no_limit(self):
-        pass
-
-    @pytest.mark.skip("Not supported by Cassandra")
-    def test_generate_content_get_range(self):
-        pass
-
-    @pytest.mark.skip("Not supported by Cassandra")
-    def test_generate_content_get_range_empty(self):
-        pass
-
-    @pytest.mark.skip("Not supported by Cassandra")
-    def test_generate_content_get_range_limit_none(self):
-        pass
-
-    @pytest.mark.skip("Not supported by Cassandra")
-    def test_generate_content_get_range_full(self):
-        pass
-
     @pytest.mark.skip("Not supported by Cassandra")
     def test_origin_count_with_visit_no_visits(self):
         pass
