@@ -47,8 +47,8 @@ def test_retrying_proxy_storage_content_add(swh_storage, sample_data):
 
     """
     sample_content = sample_data.content
-    content = next(swh_storage.content_get([sample_content.sha1]))
-    assert not content
+    content = swh_storage.content_get_data(sample_content.sha1)
+    assert content is None
 
     s = swh_storage.content_add([sample_content])
     assert s == {
@@ -56,8 +56,8 @@ def test_retrying_proxy_storage_content_add(swh_storage, sample_data):
         "content:add:bytes": sample_content.length,
     }
 
-    content = next(swh_storage.content_get([sample_content.sha1]))
-    assert content["sha1"] == sample_content.sha1
+    content = swh_storage.content_get_data(sample_content.sha1)
+    assert content == sample_content.data
 
 
 def test_retrying_proxy_storage_content_add_with_retry(
@@ -78,8 +78,8 @@ def test_retrying_proxy_storage_content_add_with_retry(
 
     sample_content = sample_data.content
 
-    content = next(swh_storage.content_get([sample_content.sha1]))
-    assert not content
+    content = swh_storage.content_get_data(sample_content.sha1)
+    assert content is None
 
     s = swh_storage.content_add([sample_content])
     assert s == {"content:add": 1}
@@ -100,8 +100,8 @@ def test_retrying_proxy_swh_storage_content_add_failure(
 
     sample_content = sample_data.content
 
-    content = next(swh_storage.content_get([sample_content.sha1]))
-    assert not content
+    content = swh_storage.content_get_data(sample_content.sha1)
+    assert content is None
 
     with pytest.raises(StorageArgumentException, match="Refuse to add"):
         swh_storage.content_add([sample_content])
@@ -117,17 +117,17 @@ def test_retrying_proxy_storage_content_add_metadata(swh_storage, sample_data):
     content = attr.evolve(sample_content, data=None)
 
     pk = content.sha1
-    content_metadata = swh_storage.content_get_metadata([pk])
-    assert not content_metadata[pk]
+    content_metadata = swh_storage.content_get([pk])
+    assert content_metadata == [None]
 
     s = swh_storage.content_add_metadata([content])
     assert s == {
         "content:add": 1,
     }
 
-    content_metadata = swh_storage.content_get_metadata([pk])
-    assert len(content_metadata[pk]) == 1
-    assert content_metadata[pk][0]["sha1"] == pk
+    content_metadata = swh_storage.content_get([pk])
+    assert len(content_metadata) == 1
+    assert content_metadata[0].sha1 == pk
 
 
 def test_retrying_proxy_storage_content_add_metadata_with_retry(
@@ -175,10 +175,6 @@ def test_retrying_proxy_swh_storage_content_add_metadata_failure(
     sample_content = sample_data.content
     content = attr.evolve(sample_content, data=None)
 
-    pk = content.sha1
-    content_metadata = swh_storage.content_get_metadata([pk])
-    assert not content_metadata[pk]
-
     with pytest.raises(StorageArgumentException, match="Refuse to add"):
         swh_storage.content_add_metadata([content])
 
@@ -485,8 +481,8 @@ def test_retrying_proxy_storage_raw_extrinsic_metadata_add(swh_storage, sample_d
     origin_metadata = swh_storage.raw_extrinsic_metadata_get(
         MetadataTargetType.ORIGIN, ori_meta.id, ori_meta.authority
     )
-    assert origin_metadata["next_page_token"] is None
-    assert not origin_metadata["results"]
+    assert origin_metadata.next_page_token is None
+    assert not origin_metadata.results
 
     swh_storage.raw_extrinsic_metadata_add([ori_meta])
 
@@ -561,9 +557,6 @@ def test_retrying_proxy_storage_directory_add(swh_storage, sample_data):
     """
     sample_dir = sample_data.directory
 
-    directory = swh_storage.directory_get_random()  # no directory
-    assert not directory
-
     s = swh_storage.directory_add([sample_dir])
     assert s == {
         "directory:add": 1,
@@ -591,9 +584,6 @@ def test_retrying_proxy_storage_directory_add_with_retry(
 
     sample_dir = sample_data.directories[1]
 
-    directory_id = swh_storage.directory_get_random()  # no directory
-    assert not directory_id
-
     s = swh_storage.directory_add([sample_dir])
     assert s == {
         "directory:add": 1,
@@ -617,9 +607,6 @@ def test_retrying_proxy_swh_storage_directory_add_failure(
 
     sample_dir = sample_data.directory
 
-    directory_id = swh_storage.directory_get_random()  # no directory
-    assert not directory_id
-
     with pytest.raises(StorageArgumentException, match="Refuse to add"):
         swh_storage.directory_add([sample_dir])
 
